/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/


/**\file**********************************************************************
 File Name	:	Control.h
 Project	:  	TIMotorLIB
 TI Doc:		na 
 \author	  	Pablo Garcia (pgarcia@isa.uniovi.es)
				Islam ElSayed (islam@isa.uniovi.es)
\brief 			Useful routines for AC machines control
\details		This module includes routines related to control of AC machines,
although is intended to be extended by any general control algorithms useful for control
or power converters. Implemented algorithms are:
    - 3ph V/Hz generation.
    - generic command shaping.
    - realizable references.
\date 			2007				
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
#ifndef CONTROL_H
#define CONTROL_H

#define MAX_COMMANDS 10		// max number of commands that can be registered

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include "Utils.h"
#include "QDOperations.h"
#include "FilterAndRotations.h"
#include "DataStorage.h"


//-----------------------------------------------------------------------------
//	Variables
//-----------------------------------------------------------------------------
/// \struct VHzConfiguration. 
/// \brief storing of V/Hz control
struct VHzConfiguration
{
	_iq twopidt;				///< 2PI*DT
	_iq PWMVCO;				///< Vmax/Fmax
	volatile struct sqddata vqds_s;		///< generated stator voltage vector
	volatile _iq voltage;			///< voltage magnitude
	volatile _iq theta;			///< voltage phase	
};

extern struct VHzConfiguration vhzConf;

/// \struct CommandsData. 
/// \brief command configuration
typedef struct CommandsData
{
    _iq vbegin;					///< initial value
    _iq vend;					///< end value
    _iq tUS;					///< time constant in us
	Uint32 actualSample;		///< counter for storing actual sample in command generation
	Uint32 finalSample;			///< counter for storing final sample in command generation
    volatile _iq* command;		///< pointer to generated command
    void* extra;				///< extra (optional) parameters required for command execution
    void (*CommandFcn)(struct CommandsData* command);	///< command function generator
    unsigned char start;		///< start/stop bit
} commandsData;

// ----------------------------------------------------------------------------

/// \struct CommandGeneratorCfg. 
/// \brief command generator configuration
typedef struct CommandGeneratorCfg
{
    unsigned int ncommand;		///< registered commands
    _iq schedPeriodUS;			///< period for command evaluation
	_iq maxCommandError;		///< maximum allowed command error
} _commandGeneratorCfg;

// ----------------------------------------------------------------------------
/// \struct CubicCommandsData. 
/// \brief For implementing Y=a*X^3+b*X^2+c*X+d;
typedef struct CubicCommandsData
{
    unsigned long count;				///< internal counter for storing actual time
    _iq tSEnd;							///< end time in Seconds
    _iq tSAct;							///< actual time in Seconds
	_iq schedPeriodS;					///< period in Seconds
    _iq derBegin;						///< begin derivative value
    _iq derEnd;							///< end derivative value
    _iq equConsta;						///< Equation constant a
    _iq equConstb;						///< Equation constant b
    _iq equConstc;						///< Equation constant c
    _iq equConstd;						///< Equation constant d
} _cubiccommandsData;

// ----------------------------------------------------------------------------


extern _commandGeneratorCfg commandGeneratorCfg;	
extern commandsData commands[MAX_COMMANDS];
//-----------------------------------------------------------------------------
//	Implemented Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Configure V/Hz control.
/// \param vbus: bus voltage (V)
/// \param fmax: max frequency (Hz)
/// \param ts: sample time (s)
//-----------------------------------------------------------------------------
extern void InitVHz(_iq vbus, _iq fmax, _iq ts);
static inline void VHz(_iq F, volatile _iq* vas, volatile _iq* vbs, 
	volatile _iq* vcs);
static inline void HomopolarInjection(volatile _iq *vas,
	volatile _iq *vbs, volatile _iq *vcs/*, const _iq Vmax*/);

//-----------------------------------------------------------------------------
//	Command shaping
//-----------------------------------------------------------------------------

static inline int RegisterCommand(_iq vbegin, _iq vend, _iq tUS, 
	volatile _iq* command, commandsData** commandData, 
	void (*CommandFcn)(struct CommandsData* command),void* extra);
static inline void StartCommands(void);
static inline void StartCommand(struct CommandsData* command);
static inline void SetCommand(struct CommandsData* command, _iq vbegin, _iq vend);
static inline void ConfigureCommandGenerator(_iq schedPeriodUS, _iq error);
static inline void StartCommandGenerator(void);
static inline void UpdateCommands(void);
static inline void UpdateCommand(struct CommandsData* command);
static inline int IsValue(struct CommandsData* command, _iq value, _iq error);
static inline int IsFinalValue(struct CommandsData* command, _iq error);

// ramp fcn
static inline void RampCommandFcn(struct CommandsData* command);

// s-function
static inline void SCommandFcn(struct CommandsData* command);
static inline void SConfigFcn(struct CommandsData* command, void* extra);
// cubic function
static inline void CubicCommandFcn(struct CommandsData* command);
static inline void CubicConfigFcn(struct CommandsData* command, void* extra,_iq derBegin,_iq derEnd);

//-----------------------------------------------------------------------------
//	Realizable references
//-----------------------------------------------------------------------------
/*static inline void RealizableReference(_iq* ref, _iq meass,
				    data error, data controlAction, _iq maxControlAction,
				    _iq kp, _iq ki);
static inline void RealizableReferenceQD(struct sqddata* ref, struct sqddata meass,
				    struct qddata* error, struct qddata* controlAction, _iq maxControlAction,
				    _iq kp, _iq ki);
*/

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	Inline function implementation
//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// V/Hz control
/// \param F: frequency (Hz)
/// \param vas: phase a voltage (V)
/// \param vbs: phase b voltage (V)
/// \param vcs: phase c voltage (V)
// ----------------------------------------------------------------------------
static inline void VHz(_iq F, volatile _iq* vas, volatile _iq* vbs, 
	volatile _iq* vcs)
{
	vhzConf.theta+=_IQmpy(F,vhzConf.twopidt);
	WRAP2PI(vhzConf.theta);
	vhzConf.voltage=_IQmpy(F,vhzConf.PWMVCO);
	SSCALAR2SYNCVEC(vhzConf.voltage,vhzConf.vqds_s,vhzConf.theta);
	DQ2PH(vhzConf.vqds_s, *vas, *vbs, *vcs);
}

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/// Hompolar harmonic injection:
	/// "Current and Flux Regulation in Field-Weakening Operation", 
	/// Fernando Briz, Alberto Diez, Michael W. Degner, Robert D. Lorenz
	/// IEEE Trans.Ind. App., Vo, 37, N 1, Jan/Feb 2001
///	\param *vas: pointer to phase a voltage
///	\param *vbs: pointer to phase b voltage
///	\param *vcs: pointer to phase c voltage
//-----------------------------------------------------------------------------
static inline void HomopolarInjection(volatile _iq *vas,
	volatile _iq *vbs, volatile _iq *vcs/*, const _iq Vmax*/)
{
	volatile _iq *vphase_max, *vphase_min;
	_iq homopolar;
	
	GetMax(vas,vbs,&vphase_max);
	GetMax(vphase_max,vcs,&vphase_max);
	GetMin(vas,vbs,&vphase_min);
	GetMin(vphase_min,vcs,&vphase_min);	
	//DV1 =  Vmax - *vphase_max;
	//DV2 = -Vmax - *vphase_min;
	//homopolar = (DV1 + DV2) >> 1;	
	homopolar = -(long)(*vphase_max + *vphase_min) >> 1;
	*vas +=homopolar;
	*vbs +=homopolar;
	*vcs +=homopolar;    
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
///    Register a new command.
///	 \param vbegin Initial value for the command
///	 \param vend Final value for the command
///	 \param tUS Time constant for the trajectory
///	 \param command Variable to be modified by the command trajectory
///	 \param commandData pointer to the registered command in the configuration structure
///	 \param CommandFcn Function pointer to the desired trajectory generator
///  \param extra (optional) parameters required for command execution
///
///	RETURNS:
///	 \return -1 if MAX_COMMANDS have been previously registered, 0 otherwise
// ----------------------------------------------------------------------------
static inline int RegisterCommand(_iq vbegin, _iq vend, _iq tUS, 
	volatile _iq* command, commandsData** commandData,
	void (*CommandFcn)(struct CommandsData* command), void* extra)
{
    if(commandGeneratorCfg.ncommand > MAX_COMMANDS)
		return -1;
    commands[commandGeneratorCfg.ncommand].vbegin = vbegin;
    commands[commandGeneratorCfg.ncommand].vend = vend;
	commands[commandGeneratorCfg.ncommand].actualSample = 0;
	commands[commandGeneratorCfg.ncommand].finalSample = _IQtoF(_IQdiv(tUS,commandGeneratorCfg.schedPeriodUS));
    commands[commandGeneratorCfg.ncommand].tUS = tUS;    
    commands[commandGeneratorCfg.ncommand].command = command;
    commands[commandGeneratorCfg.ncommand].extra = 0;
    commands[commandGeneratorCfg.ncommand].CommandFcn = CommandFcn;
	commands[commandGeneratorCfg.ncommand].extra = extra;
    commands[commandGeneratorCfg.ncommand].start = 0;    
    
    *commandData = &(commands[commandGeneratorCfg.ncommand]);
    
    commandGeneratorCfg.ncommand++;
    return 0;
    
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Start all the commands. It sets the start bit and the initial value of the variable to vbegin
// ----------------------------------------------------------------------------
static inline void StartCommands(void)
{
	int cc;
    for (cc=0;cc<commandGeneratorCfg.ncommand;cc++){
		commands[cc].start = 1;
		*(commands[cc].command) = commands[cc].vbegin;	
    }
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Start a command. It sets the start bit and the initial value of the variable to vbegin
///	 \param command pointer to the registered command in the configuration structure
// ----------------------------------------------------------------------------
static inline void StartCommand(struct CommandsData* command)
{
    command->start = 1;    
    *(command->command) = command->vbegin;
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Restarts a command. It sets the start bit and the initial value of the variable to vbegin
///	 \param command pointer to the registered command in the configuration structure
///	 \param vbegin initial value
///	 \param vend final value
// ----------------------------------------------------------------------------
static inline void SetCommand(struct CommandsData* command, _iq vbegin, _iq vend)
{
	if( (IsValue(command,vend,commandGeneratorCfg.maxCommandError)) || 
		(command->start == 1) )
		return;
    command->start = 1;    
	command->vbegin = vbegin;
	command->vend = vend;
	command->actualSample = 0;
    *(command->command) = vbegin;
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Configure command generator by reseting the ncommand variable and seting the period for the command evaluation
///	\param schedPeriodUS Period for command calculation in us.
///	\param error Maximum allowed command error
// ----------------------------------------------------------------------------
static inline void ConfigureCommandGenerator(_iq schedPeriodUS, _iq error)
{
    commandGeneratorCfg.ncommand = 0;
    commandGeneratorCfg.schedPeriodUS = schedPeriodUS;
	commandGeneratorCfg.maxCommandError = error;
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Start command generator by modifying the ISR of the selected timer
/// \deprecated
// ----------------------------------------------------------------------------
static inline void StartCommandGenerator(void)
{
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Update commands by calling the CommandFcn set during the configuration process
// ----------------------------------------------------------------------------
static inline void UpdateCommands(void)
{
	int cc;
    for (cc=0;cc<MAX_COMMANDS;cc++){
		UpdateCommand(&commands[cc]);
    }	    
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Update command by calling the CommandFcn set during the configuration process
///	 \param command pointer to the registered command in the configuration structure
// ----------------------------------------------------------------------------
static inline void UpdateCommand(struct CommandsData* command)
{
    if (command->start == 1){
    	//if(!IsFinalValue(command,commandGeneratorCfg.maxCommandError))
		if((command->actualSample)++ != command->finalSample)
			command->CommandFcn(command);
		else{
			*command->command = command->vend;
			command->start = 0;
		}
	}
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Indicate where a command has reached a value within an error
///	\param command pointer to the registered command in the configuration structure
///	\param value
///	\param error distance to the final value
///	\return 1 if final value is reached, 0 otherwise
// ----------------------------------------------------------------------------
static inline int IsValue(struct CommandsData* command, _iq value, _iq error)
{
	return (_IQabs(*(command->command) - value) <= error);
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Indicate where a command has reached the final value within an error
///	 \param command pointer to the registered command in the configuration structure
///	\param error distance to the final value
///	\return 1 if final value is reached, 0 otherwise
// ----------------------------------------------------------------------------
static inline int IsFinalValue(struct CommandsData* command, _iq error)
{
	return IsValue(command, command->vend, error);
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Generate ramp command
/// \param command pointer to command variable
// ----------------------------------------------------------------------------
static inline void RampCommandFcn(struct CommandsData* command)
{
    /**command->command += _IQdiv(_IQmpy((command->vend-command->vbegin),
    							commandGeneratorCfg.schedPeriodUS),
    							command->tUS);*/

	*command->command += (command->vend-command->vbegin)/command->finalSample;
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	Implement spline function Y=a*X^3+b*X^2+c*X+d; in case of the derivative 
///	at begin and end are equal to zero.
///	 \param command pointer to structure of type commandsData
// ----------------------------------------------------------------------------
static inline void SCommandFcn(struct CommandsData* command)
{
	struct CubicCommandsData* sCommand = (struct CubicCommandsData*)(command->extra);	
	sCommand->count++;
	sCommand->tSAct=sCommand->schedPeriodS*sCommand->count;
//	Y=a*X^3+b*X^2+c*X+d;

	*command->command	=_IQmpy(sCommand->equConsta,_IQmpy(sCommand->tSAct,_IQmpy(sCommand->tSAct,sCommand->tSAct)))
						+_IQmpy(sCommand->equConstb,_IQmpy(sCommand->tSAct,sCommand->tSAct))
		//				+_IQmpy(sCommand->equConstc,sCommand->tSAct)
						+sCommand->equConstd;

//	Y+=3*a*X.^2+2*b*X+c;
// Have numerical problems
/*
	*command->command	+=_IQmpy(scommand->equconsta*3,_IQmpy(scommand->tSact,_IQmpy(scommand->tSact,scommand->schedPeriodS)))
						+_IQmpy(scommand->equconstb*2,_IQmpy(scommand->tSact,scommand->schedPeriodS));
//						+_IQmpy(scommand->equconstc,scommand->schedPeriodS);
*/
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	calculate spline function constants in the case of derivatives at begin and 
///	end are equal to zero. d=y1; c=d1=0; a=(y2-y1)/(-0.5*x2^3); b=(-3*a*x2)/2;
///	 \param command pointer to structure of type commandsData
///	 \param extra 	pointer to structure of type CubicCommandsData
// ----------------------------------------------------------------------------
static inline void SConfigFcn(struct CommandsData* command, void* extra)
{
	_cubiccommandsData* sCommand = (_cubiccommandsData*)(extra);
	sCommand->count=0;
	sCommand->tSEnd=command->tUS/1000000;
	sCommand->schedPeriodS=commandGeneratorCfg.schedPeriodUS/1000000;

	sCommand->equConstd=command->vbegin;										//d=y1;
	sCommand->equConstc=0;														//c=d1;
	sCommand->equConsta=_IQdiv((command->vend-command->vbegin),
							_IQmpy(_IQmpy(_IQmpy(
							sCommand->tSEnd,sCommand->tSEnd)
							,sCommand->tSEnd),_IQ(-0.5)));						//a=(y2-y1)/(-0.5*x2^3);
	sCommand->equConstb=_IQmpy(_IQ(-1.5),
							_IQmpy(sCommand->equConsta,sCommand->tSEnd));	//b=(-3*a*x2)/2;
}

// --------------------------------------------------

// ----------------------------------------------------------------------------
///	Implement cubic function Y=a*X^3+b*X^2+c*X+d; with given derivatives at begin and end.
///	 \param command pointer to structure of type commandsData
// ----------------------------------------------------------------------------

static inline void CubicCommandFcn(struct CommandsData* command)
{
	struct CubicCommandsData* cubicCommand = (struct CubicCommandsData*)(command->extra);
	cubicCommand->count++;
	cubicCommand->tSAct=cubicCommand->schedPeriodS*cubicCommand->count;
//	Y=a*X^3+b*X^2+c*X+d;
	*command->command	=_IQmpy(cubicCommand->equConsta,_IQmpy(cubicCommand->tSAct,_IQmpy(cubicCommand->tSAct,cubicCommand->tSAct)))
						+_IQmpy(cubicCommand->equConstb,_IQmpy(cubicCommand->tSAct,cubicCommand->tSAct))
						+_IQmpy(cubicCommand->equConstc,cubicCommand->tSAct)
						+cubicCommand->equConstd;

//	Y+=3*a*X.^2+2*b*X+c;
// have numerical problems
/*	*command->command	+=_IQmpy(cubiccommand->equconsta*3,_IQmpy(cubiccommand->tSact,_IQmpy(cubiccommand->tSact,cubiccommand->schedPeriodS)))
						+_IQmpy(cubiccommand->equconstb*2,_IQmpy(cubiccommand->tSact,cubiccommand->schedPeriodS))
						+_IQmpy(cubiccommand->equconstc,cubiccommand->schedPeriodS);						
*/
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
///	calculate cubic function constants.
/// d=y1; c=d1; a=(y2-y1-d1*x2-(d2*x2)/2+(d1*x2)/2)/(-0.5*x2^3); b=(d2-d1-3*a*x2^2)/(2*x2);
///	 \param command pointer to structure of type commandsData
///	 \param extra 	pointer to structure of type CubicCommandsData
///	 \param derBegin at begin] 
///	 \param derEnd at end] 
// ----------------------------------------------------------------------------
static inline void CubicConfigFcn(struct CommandsData* command, void* extra,_iq derBegin,_iq derEnd)
{
	_cubiccommandsData* cubicCommand = (_cubiccommandsData*)(extra);
	cubicCommand->count=0;
	cubicCommand->tSEnd=command->tUS/1000000;
	cubicCommand->schedPeriodS=commandGeneratorCfg.schedPeriodUS/1000000;
	cubicCommand->derBegin=derBegin;
	cubicCommand->derEnd=derEnd;
	cubicCommand->equConstd=command->vbegin;										//d=y1;
	cubicCommand->equConstc=cubicCommand->derBegin;														//c=d1;
	cubicCommand->equConsta=_IQdiv(command->vend-command->vbegin-_IQmpy((long)(cubicCommand->derBegin+cubicCommand->derEnd)>>1,cubicCommand->tSEnd)
							,_IQmpy(_IQmpy(_IQmpy(
							cubicCommand->tSEnd,cubicCommand->tSEnd)
							,cubicCommand->tSEnd),_IQ(-0.5)));
	cubicCommand->equConstb=_IQdiv((cubicCommand->derEnd-cubicCommand->derBegin-
							_IQmpy(cubicCommand->equConsta*3,
							_IQmpy(cubicCommand->tSEnd,cubicCommand->tSEnd))),
							(long)cubicCommand->tSEnd<<1);
}


// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/// 	Realizable references for data variables.
///	Control action is saturated to maxControlAction
///	\param ref pointer to actual controller reference
///	\param meass meassured variable
///	\param error pointer to actual controller input error
///	\param controlAction pointer to actual control action
///	\param maxControlAction maximum allowed control action
///	\param kp proportional gain of the controller
///	\param ki integral gain of the controller
//-----------------------------------------------------------------------------
/*static inline void RealizableReference(_iq* ref, _iq meass,
				    data error, data controlAction, _iq maxControlAction,
				    _iq kp, _iq ki)
{
	if(_IQabs(controlAction[N])>= maxControlAction ){
		controlAction[N] = maxControlAction;
		// calculate error backwards
		INV_PI_REG(error,controlAction,kp,ki);
		// realizable reference
		*ref = error[N] + meass;
		// error backwards
		error[N] = *ref - meass;
	}
}*/

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// \def RealizableReference(ref, meass, error, controlAction, maxControlAction, kp, ki)
/// 	Realizable references for data variables.
///	Control action is saturated to maxControlAction
///	\param ref pointer to actual controller reference
///	\param meass meassured variable
///	\param error pointer to actual controller input error
///	\param controlAction pointer to actual control action
///	\param maxControlAction maximum allowed control action
///	\param kp proportional gain of the controller
///	\param ki integral gain of the controller
/// \note This macro can not be implemted as a static inline function because compiler
/// is not inlining it.
//-----------------------------------------------------------------------------
#define RealizableReference(ref, meass, error, controlAction, maxControlAction, kp, ki){\
	if(_IQabs(controlAction[N])>= maxControlAction ){\
		controlAction[N] = _IQmpy(maxControlAction,_IQ(SIGN(controlAction[N])));\
		INV_PI_REG(error,controlAction,kp,ki);\
	}\
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// 	Realizable references for dq variables.
///	Control action vector is saturated to maxControlAction by saturating the
///	magnitude and keeping the same phase.
///	\param ref pointer to actual controller reference
///	\param meass measured variable
///	\param error pointer to actual controller input error
///	\param controlAction pointer to actual control action
///	\param maxControlAction maximum allowed control action
///	\param kp proportional gain of the controller
///	\param ki integral gain of the controller
//-----------------------------------------------------------------------------
/*static inline void RealizableReferenceQD(struct sqddata* ref, struct sqddata meass,
				    struct qddata* error, struct qddata* controlAction, _iq maxControlAction,
				    _iq kp, _iq ki)
{
	if(MAG_QD_NDATA((*controlAction),N)>= maxControlAction ){
		// minimum phase distortion LIM
	    MODARG2QD_NDATA(maxControlAction,PHASE_QD_NDATA((*controlAction),N),(*controlAction),N); 
		// calculate error backwards
		INV_PI_QD_REG((*error),(*controlAction),kp,ki);
		// realizable reference
		ref->d = error->d[N] + meass.d;
		ref->q = error->q[N] + meass.q;
		// error backwards
		error->d[N] = (ref->d - meass.d);
		error->q[N] = (ref->q - meass.q);
	}
}*/

//-----------------------------------------------------------------------------

/// \def RealizableReferenceQD(ref, meass, error, controlAction, maxControlAction, kp, ki)
/// 	Realizable references for dq variables.
///	Control action vector is saturated to maxControlAction by saturating the
///	magnitude and keeping the same phase (minimum phase distortion LIM).
///	\param ref pointer to actual controller reference
///	\param meass measured variable
///	\param error pointer to actual controller input error
///	\param controlAction pointer to actual control action
///	\param maxControlAction maximum allowed control action
///	\param kp proportional gain of the controller
///	\param ki integral gain of the controller
/// \note This macro can not be implemted as a static inline function because compiler
/// is not inlining it.
#define RealizableReferenceQD(ref, meass, error, controlAction, maxControlAction, kp, ki){\
	if(MAG_QD_NDATA(controlAction,N)>= maxControlAction ){\
	    MODARG2QD_NDATA(maxControlAction,PHASE_QD_NDATA(controlAction,N),controlAction,N); \
		INV_PI_QD_REG(error,controlAction,kp,ki);\
	}\
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------


