/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	DataStorage.h
 Project	:  	TIMotorLIB
 
 TI Doc:		<a href="http://focus.ti.com/lit/ds/sprs439h/sprs439h.pdf"> SPRS439</a> 
 \author	:  	Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief 		Structs, Functions and macros used for storing variables in memory.
 \details		This module allows to store data in external (XINTF) or flash memory (\todo). 
 Implemented functions include write of Uint16, _iq and float data.
 \date 			2007
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef DATASTORAGE_H
#define DATASTORAGE_H

#define N	0
#define N1	1
#define N2  2

#define DECLARE_DATA(varName)    volatile data varName    = {0,0}
#define DECLARE_DATA3(varName)   volatile data3 varName   = {0,0,0}
#define DECLARE_SQDDATA(varName) volatile struct sqddata varName = {0,0}
#define DECLARE_QDDATA(varName)  volatile struct qddata varName  = {0,0,0,0}

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include "DSP2833x_Device.h"
#include "IQConstants.h"

//-----------------------------------------------------------------------------
//	Variables
//-----------------------------------------------------------------------------

//length three array for both filter coeff & data
typedef _iq data[2];	///< memory[2]
typedef _iq data3[3];	///< memory[3]
//create structure of arrays for data storage

struct sqddata {_iq q,d;};	///< complex variable
struct qddata { data q,d;};	///< complex variable with memory[2]
struct qddata3 { data3 q,d;};	///< complex variable with memory[3]



//define data initialization macros.......................................
/// initializes complex variable
#define INIT_QD(z)			z.q=_IQ(0.0); z.d=_IQ(0.0)
/// initializes memory[2]
#define INIT_DATA(z)    		z[N]=0.0; z[N1]=0.0
/// initializes memory[3]
#define INIT_DATA3(z)    		z[N]=0.0; z[N1]=0.0; z[N2]=0.0
/// initializes complex variable with memory[2]
#define INIT_QDDATA(z)   		INIT_DATA(z.q); INIT_DATA(z.d)
/// initializes complex variable with memory[3]
#define INIT_QDDATA3(z)   		INIT_DATA3(z.q); INIT_DATA3(z.d)
/// update memory[2] (n1=n)
#define UPDATE_STATE(v)			v[N1]=v[N]
/// update memory[3] (n2=n1, n1=n)
#define UPDATE_STATE3(v)		v[N2]=v[N1]; v[N1]=v[N]
/// update complex variable with memory[2]
#define UPDATEQD_STATE(vqd)		UPDATE_STATE(vqd.q);UPDATE_STATE(vqd.d)		
/// update complex variable with memory[3]
#define UPDATEQD_STATE3(vqd)	UPDATE_STATE3(vqd.q);UPDATE_STATE3(vqd.d)

// 
/// external RAM memory. Memory map is in sprs439.pdf
#define XINTF_SIZE 0x01F400// 0x00FC00	// 0x20000 (128K)

/// internal flash memory: FLASH sectors in sprs439 pp 39
#define ROM_SIZE 0x3ff80

typedef enum {
		OVERWRITE, 	///< circular overwrite
		NOVERWRITE	///< stop writing when end is reached
	    } STORAGEMODE;


// use external memory for data storage
#ifdef USE_XINTF
	#undef USE_FLASH
	#define BUFSIZE XINTF_SIZE
	#define MEM_STORAGE_DEFINED
#endif

// use flash memory for data storage
#ifdef USE_FLASH
	#undef USE_XINTF
	#define BUFSIZE ROM_SIZE
	#define MEM_STORAGE_DEFINED
#endif

#ifndef MEM_STORAGE_DEFINED
	#error MEM_STORAGE_DEFINED is not defined in DataStorage.h
#endif

/// \todo: PABLO: These variables should be in a structure
extern Uint16 dataStorageBuffer[BUFSIZE];
extern Uint16* dataPtr;
extern Uint16* begin;									
extern Uint16* end;
extern Uint32* actualPos;
								
extern Uint16 clearDataStorage;
extern STORAGEMODE storageMode;

//-----------------------------------------------------------------------------
// Implemented Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Initializes Buffer to hold data into internal memory
/// \param mode: Data storage can be configured to OVERWRITE, NOVERWRITE.
///	In OVERWRITE mode, the writing mechanism is restarted once the buffer
///	end is reached, In NOVERWRITE the writing is stopped at the end buffer.
//-----------------------------------------------------------------------------
extern void InitDataStorageBuffer(STORAGEMODE mode);


//-----------------------------------------------------------------------------
/// Write Uint16 data to internal memory
/// \param data: value to be written
//-----------------------------------------------------------------------------
static inline void WriteData(volatile Uint16 *data);

//-----------------------------------------------------------------------------
/// Write IQ value to internal memory
/// \param data: value to be written
//-----------------------------------------------------------------------------
static inline void WriteIQData(volatile _iq *data);

//-----------------------------------------------------------------------------
/// Reads Uint16 data from internal memory
/// \param data: pointer to variable to store the read data
/// \param pos: memory position holding the data
//-----------------------------------------------------------------------------
static inline void ReadData(Uint16* data,Uint32 pos);

//-----------------------------------------------------------------------------
/// Restart writing pointer to the starting address of the internal buffer
//-----------------------------------------------------------------------------
static inline void ReinitDataStorageBuffer(void);

//-----------------------------------------------------------------------------
/// Write float value to internal memory
/// \param data: value to be written
//-----------------------------------------------------------------------------
static inline void WriteFloatData(volatile float *data); //Islam

//-----------------------------------------------------------------------------

static inline void WriteFloatData(volatile float *data)
{
	
	Uint32 NumFinal = 0;
	Uint16 *ptr = (Uint16 *)data;
	
	NumFinal=NumFinal|((Uint32)(*(ptr+1))&0xFFFF)<<16;
	NumFinal=NumFinal|((Uint32)(*(ptr))&0xFFFF);

	//dataSample=(dataSample++)%BUFSIZE;
	if(dataPtr>=end-2 && storageMode == NOVERWRITE)
		return; 
	if(dataPtr>=end-2 && storageMode == OVERWRITE)
		dataPtr=begin;

	*dataPtr++ = (Uint16) (((NumFinal) & 0xFFFF0000) >> 16);
	*dataPtr++ = (Uint16) ((NumFinal) & 0x0000FFFF);


	if(storageMode == OVERWRITE)
		*actualPos = (Uint32)dataPtr;
}

//-----------------------------------------------------------------------------

static inline void WriteData(volatile Uint16 *data)
{
	//dataSample=(dataSample++)%BUFSIZE;
	if(dataPtr>=end && storageMode == NOVERWRITE)
		return; 
	if(dataPtr>=end && storageMode == OVERWRITE)
		dataPtr=begin;
	*dataPtr++= *data;
	if(storageMode == OVERWRITE){
		*actualPos = (Uint32)dataPtr;
	}
	
}

//-----------------------------------------------------------------------------


static inline void WriteIQData(volatile _iq *data)
{
	if(dataPtr>=end-2 && storageMode == NOVERWRITE)
		return;
	if(dataPtr>=end-2 && storageMode == OVERWRITE)
		dataPtr=begin;
	*dataPtr++ = (Uint16) (((long)(*data) & 0xFFFF0000) >> 16); // CCS4.0
	*dataPtr++ = (Uint16) ((long)(*data) & 0x0000FFFF); // CCS4.0

	if(storageMode == OVERWRITE){
		*actualPos = (Uint32)dataPtr;
	}		
}

//-----------------------------------------------------------------------------

static inline void ReadData(Uint16* data,Uint32 pos)
{
	if(pos<BUFSIZE)
		*data=dataStorageBuffer[pos];
}

//-----------------------------------------------------------------------------

static inline void ReinitDataStorageBuffer(void)
{
	begin=dataStorageBuffer;
	end=dataStorageBuffer+BUFSIZE-1;
	dataPtr=begin;
	clearDataStorage=0;
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

