/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name      :       Epwm.h
 Project        :       TIMotorLIB
 
  TI Doc:               <a href="http://www.ti.com/litv/pdf/spru791f"> SPRU791</a>, 
                        <a href="http://www.ti.com/litv/pdf/sprug04a"> SPRUG04</a> (newer), 
                        spraai1
 \author                Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief                 PWM Interface Routines
 \date 			2007 
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef EPWM_H
#define EPWM_H

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)
#include "SFO_V5.h"
#include GENERATE_FLEX_INC(ePwm_defines.h)
#include "IQConstants.h"
#include "Utils.h"

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
// values of interrupts ACK's on each pwm module
#define EPWM1_INT_ACK	(0x1);
#define EPWM2_INT_ACK	(0x2);
#define EPWM3_INT_ACK	(0x4);
#define EPWM4_INT_ACK	(0x8);
#define EPWM5_INT_ACK	(0x10);
#define EPWM6_INT_ACK	(0x20);

/// number of pwm outputs
#define PWM_OUTPUTS_SIZE 	(12)


//-----------------------------------------------------------------------------
//	typedefs
//-----------------------------------------------------------------------------
/// select EPWM device
typedef enum {
	EPWMA, 	///< EPWMA
	EPWMB	///< EPWMB
    } EPWMX;
    
/// select device as master or slave
typedef enum {
	MASTERPWM, 	///< EPWM is master
	SLAVEPWM	///< EPWM is slave
    } PHSEN;
    
/// access to pwm devices individually
typedef enum {
	PWM1=0,		///< EPWM1A/1B 
	PWM2=1, 	///< EPWM2A/2B
	PWM3=2, 	///< EPWM3A/3B
	PWM4=3, 	///< EPWM4A/4B
	PWM5=4, 	///< EPWM5A/5B
	PWM6=5		///< EPWM6A/6B
    } ePWMenum;
    
/// select normal mode or high resolution mode
typedef enum {
	NORMAL_MODE, 	///< normal mode
	HRPWM_MODE	///< high resolution mode
    } PWMMODE;
    
/// select carrier signal (sawtooth or triangle)
typedef enum {
	SAWTOOTH=TB_COUNT_UP,			///< sawtooth carrier
        TRIANGLE=TB_COUNT_UPDOWN                ///< triangle carrier
        } PWMCARRIER;
        
/// select modulation (symmetrical, asymmetrical) for triangle pwm carrier
typedef enum {
        SYMMETRICAL=CC_CTR_ZERO,                ///< symmetrical modulation 
        ASYMMETRICAL=CC_CTR_ZERO_PRD    ///< asymmetrical modulation (update on zero and period)
} PWMMODULATION;

/// interrupt generation
typedef enum {
	CTR_ZERO=ET_CTR_ZERO,			///< interrupt on counter reach zero
	CTR_PRD=ET_CTR_PRD				///< interrupt on counter reach period
} INTSEL;

/// start of conversion
typedef enum {
        SOCNULL=0,      ///< A/D start of conversion decoupled from EPWM
        SOCA=1,         ///< EPWM generates SOC for sequencer A
        SOCB=2          ///< EPWM generates SOC for sequencer B
    } ePWMSOC;
    
/// triggers
typedef enum {
	EvTBeqZERO=1,		///< trigger A/D on zero
	EvTBeqPRD=2, 		///< trigger A/D of period
	EvTBCNTeqCMPAI=4,	///< trigger A/D on compare math EPWMA (increasing)
	EvTBCNTeqCMPAD=5,	///< trigger A/D on compare math EPWMA (decreasing)
	EvTBCNTeqCMPBI=6,	///< trigger A/D on compare math EPWMB (increasing)
	EvTBCNTeqCMPBD=7	///< trigger A/D on compare math EPWMB (decreasing)
    } SOCTrigger;
    
/// select top or bottom switch (A --> TOP, B --> BOTTOM)
typedef enum {
	TOP=0, 			///< TOP switches
        BOTTOM=6                ///< BOTTOM switches    
    } sideEnum;

/// select switch logic
typedef enum {
        NEG_LOGIC=1,
        POS_LOGIC=2
} SWITCHLOGIC;


//-----------------------------------------------------------------------------
//      variables
// ----------------------------------------------------------------------------

/// three phase PWM configuration
struct PWM3phConfiguration {
	Uint16 t_pwm;			///< period (us)
	Uint16 OFFSETPWM;		///< period/2
	_iq OFFSETPWMDIVVBUS;		///< period/2/vbus
	_iq vBus;			///< bus voltage
	EPWMX epwmX;			///< select EPWMA, EPWMB
	volatile Uint16* cmp1;		///< compare leg 1
	volatile Uint16* cmp2;		///< compare leg 2
	volatile Uint16* cmp3;		///< compare leg 3
	volatile Uint16* enable_input;	///< enable/disable input
	Uint16 enabled;			///< enable/disable modulation \deprecated
};

/// \deprecated: Not use pwmConf in new projects. Instead, pwmAConf or pwmBConf
extern volatile struct PWM3phConfiguration pwmConf;
extern volatile struct PWM3phConfiguration pwmAConf, pwmBConf;

/// acess pointers to pwm configuration registers
extern volatile struct EPWM_REGS *ePWM[];
/// fast access to EPWM comparison registers
extern volatile Uint16* cmpPtr[PWM_OUTPUTS_SIZE];
extern int MEP_ScaleFactor[PWM_CH];
extern int MEP_SF1, MEP_SF2, MEP_SF3, MEP_SF4, MEP_SF5, MEP_SF6;

//-----------------------------------------------------------------------------
// Declaration of implemented Functions
//-----------------------------------------------------------------------------

// configuration & init function
extern void BindPWMOutputs(void);

//-----------------------------------------------------------------------------
/// Init clocks on PWM inputs EPWMA/B depending on the configuration stored in pwmConf.epwmX
//-----------------------------------------------------------------------------
extern void InitPWMPeripheralClocks(void);

//-----------------------------------------------------------------------------
/// Init clocks on PWM inputs EPWMA
//-----------------------------------------------------------------------------
extern void InitPWMAPeripheralClocks(void);

//-----------------------------------------------------------------------------
/// Init clocks on PWM inputs EPWMB
//-----------------------------------------------------------------------------
extern void InitPWMBPeripheralClocks(void);

//-----------------------------------------------------------------------------
/// Configure needed GPIO ports to be used by EPWM outputs.
/// Affected GPIO inputs are:
///	EPWM1: GPIO0, GPIO1
///	EPWM2: GPIO2, GPIO3
///	EPWM3: GPIO4, GPIO5
///	EPWM4: GPIO6, GPIO7
///	EPWM5: GPIO8, GPIO9
///	EPWM6: GPIO10, GPIO11
/// Actions taken:
///	- Enable internal pull-up (GpioCtrlRegs.GPAPUD.bit.GPIOX=0)
///	- Configure GPIOX as EPWM (GpioCtrlRegs.GPAMUX1.bit.GPIOX=1)
/// \param PWM is the pwm input to be used
//-----------------------------------------------------------------------------
extern void InitEPWMGpio(ePWMenum PWM);

//-----------------------------------------------------------------------------
/// Configure needed GPIO ports to be used by EPWM Trip-zone submodule.
/// Affected GPIO inputs are:
///	TZ1: GPIO12
///	TZ2: GPIO13
///	TZ3: GPIO14
///	TZ4: GPIO15
///	TZ5: GPIO16
///	TZ6: GPIO17
/// Actions taken:
///	- Enable internal pull-up (GpioCtrlRegs.GPAPUD.bit.GPIOX=0)
///	- Asynch input GPIOX (GpioCtrlRegs.GPAQSEL1.bit.GPIOX = 3)
///	- Configure GPIOX as TZX (GpioCtrlRegs.GPAMUXn.bit.GPIOX = 1)
//-----------------------------------------------------------------------------
extern void InitTzGpio(void);

//-----------------------------------------------------------------------------
/// Initializes a given EPWM
/// \param interruptFCN: pointer to ISR executed when intSel condition is
///     reached. If 0, no function is called
/// \param PWM: Selection of PWM input to be configured (PWM1...PWM6)
/// \param tpwm_us: switching period in us
/// \param pwmMode: normal mode (16 bits) or high resolution (\todo) (32 bits) 
/// \param pwmCarrier: select SAWTOOTH or TRIANGLE carrier
/// \param modulation: SYMMETRICAL/ASYMMETRICAL. If symmetrical, comparison
///     values are updated when the counter reaches 0. If asymmetrical, comparison
///     values are updated both at 0 and period of the counter
/// \param intSel: CTR_ZERO/CTR_PRD. Call the ISR when the counter reach 0 or
///     when the counter reach the period.
/// \param switchLogic: POS_LOGIC/NEG_LOGIC. Select logic for the drived switch
//-----------------------------------------------------------------------------
extern void InitSinglePWMOutput(void(*interruptFCN)(void), ePWMenum PWM, Uint16 tpwm_us, 
        PWMMODE pwmMode, PWMCARRIER pwmCarrier, PWMMODULATION modulation, INTSEL intSel,
        SWITCHLOGIC switchLogic);

//-----------------------------------------------------------------------------
/// Intializes a given EPWM pair in complementary mode
/// \param interruptFCN: pointer to ISR executed when intSel condition is
///     reached. If 0, no function is called
/// \param PWM: Selection of PWM input to be configured (PWM1...PWM6)
/// \param tpwm_us: switching period in us
/// \param db_us: dead time value in us
/// \param pwmMode: normal mode (16 bits) or high resolution (\todo) (32 bits) 
/// \param pwmCarrier: select SAWTOOTH or TRIANGLE carrier
/// \param modulation: SYMMETRICAL/ASYMMETRICAL. If symmetrical, comparison
///     values are updated when the counter reachs 0. If asymmetrical, comparison
///     values are updated both at 0 and period of the counter
/// \param intSel: CTR_ZERO/CTR_PRD. Call the ISR when the counter reach 0 or
///     when the counter reach the period.
/// \param switchLogic: POS_LOGIC/NEG_LOGIC. Select logic for the switches
//-----------------------------------------------------------------------------
extern void InitComplementaryPWMOutput(void(*interruptFCN)(void), ePWMenum PWM, Uint16 tpwm_us,
        Uint16 db_us,  PWMMODE pwmMode, PWMCARRIER pwmCarrier, PWMMODULATION modulation, 
        INTSEL intSel, SWITCHLOGIC switchLogic);


//-----------------------------------------------------------------------------
/// Intializes a set of EPWM outputs to drive a three phase inverter
/// \param interruptFCN: pointer to ISR executed when intSel condition is
///     reached. If 0, no function is called
/// \param epwmX: Selection of PWM input to be configured (PWM1...PWM6)
/// \param tpwm_us: switching period in us
/// \param db_us: dead time value in us
/// \param pwmCarrier: select SAWTOOTH or TRIANGLE carrier
/// \param modulation: SYMMETRICAL/ASYMMETRICAL. If symmetrical, comparison
///     values are updated when the counter reaches 0. If asymmetrical, comparison
///     values are updated both at 0 and period of the counter
/// \param intSel: CTR_ZERO/CTR_PRD. Call the ISR when the counter reach 0 or
///     when the counter reach the period. 
/// \param vBus: DC bus voltage in V (\todo: Eliminate this parameter)
/// \param soc: SOCNULL/SOCA/SOCB. Generates a start of conversion trigger for
///     none, A or B sequencer respectively
/// \param switchLogic: POS_LOGIC/NEG_LOGIC. Select logic for the switches
//-----------------------------------------------------------------------------
extern void Init3phPWM(void(*interruptFCN)(void), EPWMX epwmX, Uint16 tpwm_us, 
        Uint16 db_us,   PWMCARRIER pwmCarrier, PWMMODULATION modulation, INTSEL intSel,
        _iq vBus, ePWMSOC soc, SWITCHLOGIC switchLogic);

//-----------------------------------------------------------------------------
/// Configure two switches to work in complementary mode
/// \param pwmPtr: Pointer to EPWM_REGS register for the configuration of the switches pair 
/// \param master: MASTERPWM/SLAVEPWM. Selects if the clocking signal is generated by the 
///	configured output or is passed externally.
/// \param tpwm_us: switching period in us
/// \param db_us: dead time value in us
/// \param pwmCarrier: select SAWTOOTH or TRIANGLE carrier
/// \param modulation: SYMMETRICAL/ASYMMETRICAL. If symmetrical, comparison
///     values are updated when the counter reaches 0. If asymmetrical, comparison
///     values are updated both at 0 and period of the counter
/// \param polSel: Polarity selection for dead time generation (DB_ACTIVE_LOC/DB_ACTIVE_HIC)
//-----------------------------------------------------------------------------
extern void ConfigurePWMLeg(volatile struct EPWM_REGS* pwmPtr, PHSEN master, 
	Uint16 tpwm_us, Uint16 db_us, PWMCARRIER pwmCarrier, PWMMODULATION modulation, 
	unsigned int polSel);

//-----------------------------------------------------------------------------
/// Selects where the Start of Conversion (SOC) signal is generated
/// \param pwmPtr: Pointer to EPWM_REGS register for the configuration
/// \param soc: Which sequencer is triggered (SOCNULL/SOCA/SOCB)
/// \param trigger:
///	- EvTBeqZERO		trigger A/D on zero
///	- EvTBeqPRD		trigger A/D of period
///	- EvTBCNTeqCMPAI	trigger A/D on compare math EPWMA (increasing)
///	- EvTBCNTeqCMPAD	trigger A/D on compare math EPWMA (decreasing)
///	- EvTBCNTeqCMPBI	trigger A/D on compare math EPWMB (increasing)
///	- EvTBCNTeqCMPBD	trigger A/D on compare math EPWMB (decreasing)
//-----------------------------------------------------------------------------
extern void AddEventTrigger(volatile struct EPWM_REGS* pwmPtr,
	ePWMSOC soc, SOCTrigger trigger);

// real time update & enable functions
static inline void SetEnablePWMInput(Uint16* input);
static inline void UpdateEPWM(ePWMenum PWM, sideEnum side, _iq cmp);
static inline void UpdateEPWMDuty(ePWMenum PWM, sideEnum side, _iq duty);
static inline void UpdateHRPWM(ePWMenum PWM, _iq cmp);
/// \deprecated: Not use UpdatePWM3ph in new projects. Instead, UpdatePWMA3ph or UpdatePWMB3ph
static inline void UpdatePWM3ph(_iq vas,_iq vbs,_iq vcs, char saturate);
static inline void UpdatePWMA3ph(_iq vas,_iq vbs,_iq vcs, char saturate);
static inline void UpdatePWMB3ph(_iq vas,_iq vbs,_iq vcs, char saturate);
/// \deprecated: Not use UpdatePWMDuty3ph in new projects. Instead, UpdatePWMADuty3ph or UpdatePWMBDuty3ph
static inline void UpdatePWMDuty3ph(_iq vad, _iq vbd, _iq vcd);
static inline void UpdatePWMADuty3ph(_iq vad, _iq vbd, _iq vcd);
static inline void UpdatePWMBDuty3ph(_iq vad, _iq vbd, _iq vcd);
static inline void ForceSingleEPWM(ePWMenum PWM, sideEnum side, Uint16 level);
static inline void _ForceSingleEPWM(volatile struct EPWM_REGS * pwmPtr, sideEnum side, 
	Uint16 level);
static inline void ForceEPWM(ePWMenum PWM, sideEnum side, Uint16 level);
static inline void EnablePWM(Uint16 enable);
static inline void EnableEPWMModuleTBCLK(void);
static inline void EpwmPIEAck(volatile struct EPWM_REGS* epwmRegs);
static inline void EpwmAPIEAck(void);
static inline void EpwmBPIEAck(void);

//-----------------------------------------------------------------------------
// Implemented Functions
//-----------------------------------------------------------------------------

static inline void SetEnablePWMInput(Uint16* input)
{
	pwmConf.enable_input = input;
}

//-----------------------------------------------------------------------------

static inline void UpdateEPWM(ePWMenum PWM, sideEnum side, _iq cmp)
{
	*(cmpPtr[PWM+side]) = (Uint16)(_IQtoF(cmp));
}

//-----------------------------------------------------------------------------

static inline void UpdateEPWMDuty(ePWMenum PWM, sideEnum side, _iq duty)
{
        *(cmpPtr[PWM+side]) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(duty),_IQ16(ePWM[PWM]->TBPRD))));
}

//-----------------------------------------------------------------------------

static inline void UpdateHRPWM(ePWMenum PWM, _iq cmp)
{
	ePWM[PWM]->CMPA.all = (_IQint(cmp*ePWM[PWM]->TBPRD) << 16) + 							// get integer part
		(((Uint32)(_IQfrac(cmp*ePWM[PWM]->TBPRD*MEP_ScaleFactor[0])) << 8) + 0x180);		// get fractional part
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// \deprecated: Use UpdatePWMA3ph or UpdatePWMB3ph instead.
//-----------------------------------------------------------------------------
static inline void UpdatePWM3ph(_iq vas, _iq vbs, _iq vcs, char saturate)
{
	// enable/disable PWM
	//EnablePWM(*(pwmConf.enable_input));
	
	// update the values for compare units 1,2,3 register
	/// \todo These operation need to be changed if 2*OFFSETPWM > MAX IQ (4096)
	/**(pwmConf.cmp1) = (Uint16)(_IQtoF(_IQmpy(vas,pwmConf.OFFSETPWMDIVVBUS))+pwmConf.OFFSETPWM);
	*(pwmConf.cmp2) = (Uint16)(_IQtoF(_IQmpy(vbs,pwmConf.OFFSETPWMDIVVBUS))+pwmConf.OFFSETPWM);
	*(pwmConf.cmp3) = (Uint16)(_IQtoF(_IQmpy(vcs,pwmConf.OFFSETPWMDIVVBUS))+pwmConf.OFFSETPWM);*/
	if(saturate){
		LIM(vas,(long)(pwmConf.vBus)>>1); // CCS4.0
                LIM(vbs,(long)(pwmConf.vBus)>>1); // CCS4.0
                LIM(vcs,(long)(pwmConf.vBus)>>1); // CCS4.0
        }
        /**(pwmConf.cmp1) = (Uint16)(pwmConf.OFFSETPWM - _IQtoF(_IQmpy(vas,pwmConf.OFFSETPWMDIVVBUS)));
        *(pwmConf.cmp2) = (Uint16)(pwmConf.OFFSETPWM - _IQtoF(_IQmpy(vbs,pwmConf.OFFSETPWMDIVVBUS)));
        *(pwmConf.cmp3) = (Uint16)(pwmConf.OFFSETPWM - _IQtoF(_IQmpy(vcs,pwmConf.OFFSETPWMDIVVBUS)));*/
        *(pwmConf.cmp1) = (Uint16)(_IQtoF(_IQmpy(vas,pwmConf.OFFSETPWMDIVVBUS)));
        *(pwmConf.cmp2) = (Uint16)(_IQtoF(_IQmpy(vbs,pwmConf.OFFSETPWMDIVVBUS)));
        *(pwmConf.cmp3) = (Uint16)(_IQtoF(_IQmpy(vcs,pwmConf.OFFSETPWMDIVVBUS)));

        // Acknowledge this interrupt to receive more interrupts from group 3
        // PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Update EPWMA configured for three phase systems
///     \param vas voltage for phase a
///     \param vbs voltage for phase b
///     \param vcs voltage for phase c
///	\param saturate saturate to Vbus/2
//-----------------------------------------------------------------------------
static inline void UpdatePWMA3ph(_iq vas, _iq vbs, _iq vcs, char saturate)
{
        /// \todo: Saturate using IQsat functions
	if(saturate){
		LIM(vas,(long)(pwmAConf.vBus)>>1); // CCS4.0
                LIM(vbs,(long)(pwmAConf.vBus)>>1); // CCS4.0
                LIM(vcs,(long)(pwmAConf.vBus)>>1); // CCS4.0
        }
        /**(pwmAConf.cmp1) = (Uint16)(pwmAConf.OFFSETPWM - _IQtoF(_IQmpy(vas,pwmAConf.OFFSETPWMDIVVBUS)));
        *(pwmAConf.cmp2) = (Uint16)(pwmAConf.OFFSETPWM - _IQtoF(_IQmpy(vbs,pwmAConf.OFFSETPWMDIVVBUS)));
        *(pwmAConf.cmp3) = (Uint16)(pwmAConf.OFFSETPWM - _IQtoF(_IQmpy(vcs,pwmAConf.OFFSETPWMDIVVBUS)));*/
        *(pwmAConf.cmp1) = (Uint16)(_IQtoF(_IQmpy(vas,pwmAConf.OFFSETPWMDIVVBUS)));
        *(pwmAConf.cmp2) = (Uint16)(_IQtoF(_IQmpy(vbs,pwmAConf.OFFSETPWMDIVVBUS)));
        *(pwmAConf.cmp3) = (Uint16)(_IQtoF(_IQmpy(vcs,pwmAConf.OFFSETPWMDIVVBUS)));
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Update EPWMB configured for three phase systems
///     \param vas voltage for phase a
///     \param vbs voltage for phase b
///     \param vcs voltage for phase c
///	\param saturate saturate to Vbus/2
//-----------------------------------------------------------------------------
static inline void UpdatePWMB3ph(_iq vas, _iq vbs, _iq vcs, char saturate)
{
    /// \todo: Saturate using IQsat functions
	if(saturate){
		LIM(vas,(long)(pwmBConf.vBus)>>1); // CCS4.0
                LIM(vbs,(long)(pwmBConf.vBus)>>1); // CCS4.0
                LIM(vcs,(long)(pwmBConf.vBus)>>1); // CCS4.0
        }
        /**(pwmBConf.cmp1) = (Uint16)(pwmBConf.OFFSETPWM - _IQtoF(_IQmpy(vas,pwmBConf.OFFSETPWMDIVVBUS)));
        *(pwmBConf.cmp2) = (Uint16)(pwmBConf.OFFSETPWM - _IQtoF(_IQmpy(vbs,pwmBConf.OFFSETPWMDIVVBUS)));
        *(pwmBConf.cmp3) = (Uint16)(pwmBConf.OFFSETPWM - _IQtoF(_IQmpy(vcs,pwmBConf.OFFSETPWMDIVVBUS)));*/
        *(pwmBConf.cmp1) = (Uint16)(_IQtoF(_IQmpy(vas,pwmBConf.OFFSETPWMDIVVBUS)));
        *(pwmBConf.cmp2) = (Uint16)(_IQtoF(_IQmpy(vbs,pwmBConf.OFFSETPWMDIVVBUS)));
        *(pwmBConf.cmp3) = (Uint16)(_IQtoF(_IQmpy(vcs,pwmBConf.OFFSETPWMDIVVBUS)));
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///	\deprecated: Use UpdatePWMADuty3ph or UpdatePWMBDuty3ph instead.
//-----------------------------------------------------------------------------
static inline void UpdatePWMDuty3ph(_iq vad, _iq vbd, _iq vcd)
{
        /*
        *(pwmConf.cmp1) = (Uint16)((pwmConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmConf.OFFSETPWM << 1)))));
        *(pwmConf.cmp2) = (Uint16)((pwmConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmConf.OFFSETPWM << 1)))));
        *(pwmConf.cmp3) = (Uint16)((pwmConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmConf.OFFSETPWM << 1)))));
        */
                *(pwmConf.cmp1) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmConf.OFFSETPWM << 1)))));
        *(pwmConf.cmp2) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmConf.OFFSETPWM << 1)))));
        *(pwmConf.cmp3) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmConf.OFFSETPWM << 1)))));
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///	update EPWMA duty for three phase configuration:
///	cmpX = 2*OFFSETPWM(1 - vXd)
///	vXd = [0,1]
///	\param vad duty for phase a
///	\param vbd duty for phase b
///	\param vcd duty for phase c
//-----------------------------------------------------------------------------
static inline void UpdatePWMADuty3ph(_iq vad, _iq vbd, _iq vcd)
{
        /**(pwmAConf.cmp1) = (Uint16)((pwmAConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
        *(pwmAConf.cmp2) = (Uint16)((pwmAConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
        *(pwmAConf.cmp3) = (Uint16)((pwmAConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
                */
                *(pwmAConf.cmp1) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
        *(pwmAConf.cmp2) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
        *(pwmAConf.cmp3) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmAConf.OFFSETPWM << 1)))));
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///	update EPWMB duty for three phase configuration:
///	cmpX = 2*OFFSETPWM(1 - vXd)
///	vXd = [0,1]
///	\param vad duty for phase a
///	\param vbd duty for phase b
///	\param vcd duty for phase c
//-----------------------------------------------------------------------------
static inline void UpdatePWMBDuty3ph(_iq vad, _iq vbd, _iq vcd)
{
        /*
        *(pwmBConf.cmp1) = (Uint16)((pwmBConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
        *(pwmBConf.cmp2) = (Uint16)((pwmBConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
        *(pwmBConf.cmp3) = (Uint16)((pwmBConf.OFFSETPWM << 1) - 
                _IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
        */
                *(pwmBConf.cmp1) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vad),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
        *(pwmBConf.cmp2) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vbd),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
        *(pwmBConf.cmp3) = (Uint16)(_IQ16toF(_IQ16mpy(_IQtoIQ16(vcd),(_IQ16(pwmBConf.OFFSETPWM << 1)))));
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Force EPWM values (spru791a.pdf, pp. 100)
//-----------------------------------------------------------------------------
static inline void ForceSingleEPWM(ePWMenum PWM, sideEnum side, Uint16 level)
{
	_ForceSingleEPWM(ePWM[PWM], side, level);
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Force Single PWM output to a given level
/// \param pwmPtr Pointer to the EPWM_REGS register which output wants to be forced.
/// \param side selection of TOP/BOTTOM output
/// \param level 0 --> low, 1 --> high
//-----------------------------------------------------------------------------
static inline void _ForceSingleEPWM(volatile struct EPWM_REGS * pwmPtr, 
	sideEnum side, Uint16 level)
{
	pwmPtr->AQSFRC.bit.RLDCSF = 3;			// load immediately
	if(side == TOP){
		pwmPtr->AQSFRC.bit.ACTSFA = level+1;
		pwmPtr->AQSFRC.bit.OTSFA=1;
	}
	else{
		pwmPtr->AQSFRC.bit.ACTSFB = level+1;
		pwmPtr->AQSFRC.bit.OTSFB=1;
	}
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Force Single PWM output to a given level
/// \param PWM output selection by identifier (PWM1..PWM6)
/// \param side selection of TOP/BOTTOM output
/// \param level 0 --> low, 1 --> high
//-----------------------------------------------------------------------------
static inline void ForceEPWM(ePWMenum PWM, sideEnum side, Uint16 level)
{
	if(side == TOP)
		ePWM[PWM]->AQCSFRC.bit.CSFA = level+1;
	else
		ePWM[PWM]->AQCSFRC.bit.CSFB = level+1;

}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// \deprecated: Not used anymore
//-----------------------------------------------------------------------------
static inline void EnablePWM(Uint16 enable)
{
	// EvaRegs.COMCONA.bit.FCOMPOE = enable;
	pwmConf.enabled = enable;
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Enable device clock
//-----------------------------------------------------------------------------
static inline void EnableEPWMModuleTBCLK(void)
{
	// enable clock
   	EALLOW;
   	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
   	EDIS;
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///     ACK interrupt for epwmRegs. This function MUST be called if more interrupts
/// from PIEACK_GROUP3 are wanted
/// \param epwmRegs pointer to EPWM_REGS register which interrupt is ACK
//-----------------------------------------------------------------------------
static inline void EpwmPIEAck(volatile struct EPWM_REGS* epwmRegs)
{
        // Acknowledge this interrupt to receive more interrupts from group 3
        PieCtrlRegs.PIEACK.all |= PIEACK_GROUP3;
        // Clear EPWMxINT flag
        epwmRegs->ETCLR.bit.INT = 1;
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///     ACK interrupt for EPWMA. This function MUST be called if more interrupts
/// from PIEACK_GROUP3 are wanted
//-----------------------------------------------------------------------------
static inline void EpwmAPIEAck(void)
{
        EpwmPIEAck(&EPwm1Regs);
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
///     ACK interrupt for EPWMB. This function MUST be called if more interrupts
/// from PIEACK_GROUP3 are wanted
//-----------------------------------------------------------------------------
static inline void EpwmBPIEAck(void)
{
        EpwmPIEAck(&EPwm4Regs);
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
