/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**********************************************************************1********/

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "adc.h"
#include "cpu.h"

//-----------------------------------------------------------------------------
// Define
//-----------------------------------------------------------------------------
#define ADC_usDELAY  5000L

struct ADConfiguration adConfiguration;

/// Calculate needed prescaler values (cps, acq_ps) for the given 
/// frequency (fAD [Mhz]) and sampling time (tSH [ns])
static inline void CalculateADClocks(_iq fAD, Uint16 tSH, volatile Uint16* adcCLKPS, 
				     volatile Uint16* cps, volatile Uint16* acq_ps)
{
    _iq hspCLKinMhz = HspCLKInMhz();
    _iq adcCLK;
    *cps = 0;
    *adcCLKPS = hspCLKinMhz/fAD >> 1;
    if ((*adcCLKPS) > 15){
		*cps = 1;
		*adcCLKPS = (*adcCLKPS) >> 1;
    }
	if((*adcCLKPS) > 0)
		adcCLK = hspCLKinMhz/((*adcCLKPS)*(*cps + 1)<<1);
	else
		adcCLK = hspCLKinMhz/(*cps + 1);
    //adcCLK = *adcCLKPS > 0 ? hspCLKinMhz/((*adcCLKPS)*(*cps + 1)<<1) : hspCLKinMhz/(*cps + 1);
    *acq_ps = (Uint16)(_IQtoF(_IQmpy(_IQ(tSH)/1000,adcCLK)));
}

//-----------------------------------------------------------------------------

void ConfigureAD(void(*interruptFCN)(void), _iq fAD, Uint16 tSH, 
	SEQ_MODE seqMode, SAMPLING_MODE samplingMode, SOC_MODE socMode,
	Uint16 offtrim)
{
	int chCnt;
	volatile Uint16 adcCLKPS, cps, acq_ps;
	InitAD();
	// configure AD clocking 
	// @DOC SPRU716B, pp 28, 34, 38
	// AD prescaler
	// 0 		ADCLK = HSPCLK/(CPS + 1)
	// ADCCLKPS(!=0)	ADCLK = HSPCLK/[2*ADCCLKPS*(CPS + 1)]
	CalculateADClocks(fAD, tSH, &adcCLKPS, &cps, &acq_ps);
		
	AdcRegs.ADCTRL3.bit.ADCCLKPS=adcCLKPS;//2;	// (4 Juama) // (0 - 0xF)	
	AdcRegs.ADCTRL1.bit.CPS=cps;//1;
	// Acquisition window size ((ACQ_PS+1)*ADCLKK --AD clock)
	AdcRegs.ADCTRL1.bit.ACQ_PS=acq_ps;//0x7; //(0xF Juamma);
	// channel list initialization and clear
	AdcRegs.ADCCHSELSEQ1.all = 0;
	AdcRegs.ADCCHSELSEQ2.all = 0;
	AdcRegs.ADCCHSELSEQ3.all = 0;
	AdcRegs.ADCCHSELSEQ4.all = 0;
	AdcRegs.ADCMAXCONV.bit.MAX_CONV1 = 0;
	AdcRegs.ADCMAXCONV.bit.MAX_CONV2 = 0;
	// cascade/dual sequence
	AdcRegs.ADCTRL1.bit.SEQ_CASC = seqMode;
	// simultaneous/sequential
	AdcRegs.ADCTRL3.bit.SMODE_SEL = samplingMode;
	// configure EOC interrupts
	// enable interrupt if ISR is specified
    if(interruptFCN != 0){
        EALLOW;  
   		PieVectTable.ADCINT = interruptFCN;
   		EDIS;
       	PieCtrlRegs.PIEIER1.bit.INTx6 = 1;	// Enable ADCINT in PIE
        IER |= M_INT1;						// Enable CPU Interrupt 1
    }
	// configure SOC event
	AdcRegs.ADCTRL2.all |= socMode;
	//AdcRegs.ADCTRL2.bit.INT_ENA_SEQ1 = 1;
	//AdcRegs.ADCTRL2.bit.EPWM_SOCA_SEQ1 = 1;
	// Enable SEQ1 interrupt (every EOS)
   	// AdcRegs.ADCTRL2.bit.INT_ENA_SEQ1 = 1;	// (not needed)  
	// Adquisition hardware registers
	AdcRegs.ADCREFSEL.bit.REF_SEL=0;	// Internal reference selected
	AdcRegs.ADCOFFTRIM.bit.OFFSET_TRIM = offtrim;
	
	// initialize AD conf structure
	adConfiguration.fAD = fAD;
	adConfiguration.tSH = tSH;	
	adConfiguration.selSeqPtr = &AdcRegs.ADCCHSELSEQ1.all;
	*(adConfiguration.selSeqPtr) = 0;
	adConfiguration.max_ch = 0;
	adConfiguration.trigger=0;
	for (chCnt=0; chCnt < MAX_CH; chCnt++)
		adConfiguration.convChannel[chCnt] = 0;

}

// ----------------------------------------------------------------------------

/*-----------------------------------------------------------------------------
	Add a channel to the channel list. 

	Parameters:
		- input: ADCINPUT that is sampled
		- channel: channel select in the converter list
		- offset: allows to specify an offset between the bits value and the units
		- gain: gain between the bits value and the units one
		val = gain*bits+offset
------------------------------------------------------------------------------*/
void AddChannel(Uint16 input, Uint16 canal, _iq offset, _iq gain)
{
	if(adConfiguration.max_ch == MAX_CH-1)
		return;
	// add channel to channel list
	adConfiguration.chlChannels[input] = canal*(SIMULTANEOUS_MODE+1) + 
		adConfiguration.convChannel[canal]++;
        // configure channel offset and gain
        adConfiguration.chlGains[input] = gain;
        adConfiguration.chlOffsets[input] = offset;
        // set default polarity (bipolar --> 2048)
        adConfiguration.chlPolarity[input] = AD_ORIGIN;
        // get the appropiate sequence number
        if( (AdcRegs.ADCTRL3.bit.SMODE_SEL == SIMULTANEOUS_MODE) && input >=ADCINB0 )
                input = input%ADCINB0;
	// add input to ADCCHSELSEQX
	// @DOC SPRU716B --> pag. 21
	(*(adConfiguration.selSeqPtr+canal/4)) |= ( (input & 0x000F) << (canal*4)%16 );
	// specify max number of conversions. It depends on sequence configuration
	if(adConfiguration.max_ch < canal){
                adConfiguration.max_ch = canal;
                AdcRegs.ADCMAXCONV.all =  adConfiguration.max_ch;// | (adConfiguration.max_ch << 4);
        }
}

// ----------------------------------------------------------------------------

void AddUnipolarChannel(Uint16 input, Uint16 channel, _iq offset, _iq gain)
{
	AddChannel(input, channel, offset, gain);
	adConfiguration.chlPolarity[input] = 0;
	
}

// ----------------------------------------------------------------------------

void InitAD(void)
{
    InitADPeripheralClocks();
    // To powerup the ADC the ADCENCLK bit should be set first to enable
    // clocks, followed by powering up the bandgap, reference circuitry, and ADC core.
    // Before the first conversion is performed a 5ms delay must be observed 
    // after power up to give all analog circuits time to power up and settle
    AdcRegs.ADCTRL3.all = 0x00E0;  // Power up bandgap/reference/ADC circuits
    DELAY_US(ADC_usDELAY);         // Delay before converting ADC channels
}

// ----------------------------------------------------------------------------

void InitADPeripheralClocks(void)
{
    EALLOW;
	SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1;    // ADC
    EDIS;
}

// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
