/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

// --------------------------------------------------------------------------
// Includes
// --------------------------------------------------------------------------
#include "cpu.h"

/*-----------------------------------------------------------------------------
    Variables
------------------------------------------------------------------------------*/
volatile struct CPUConfiguration cpuConf;


void InitCPU(Uint32 oscCLK, Uint16 dsp28_DIVSEL, Uint16 dsp28_PLLCR,
	Uint16 hspCLK, Uint16 lspCLK)
{
	cpuConf.oscCLK = oscCLK;
	cpuConf.dsp28_DIVSEL = dsp28_DIVSEL;
	cpuConf.dsp28_PLLCR =dsp28_PLLCR; 
	cpuConf.hspCLK = hspCLK;
	cpuConf.lspCLK = lspCLK;
    InitSysCtrl();
    // Initialize the PIE control registers to their default state.
    // The default state is all PIE interrupts disabled and flags
    // are cleared.
    InitPieCtrl();
    // Disable CPU interrupts and clear all CPU interrupt flags:
   	IER = 0x0000;
   	IFR = 0x0000;
	// Initialize the PIE vector table with pointers to the shell Interrupt
	// Service Routines (ISR).  
   	// InitPieVectTable();
}

//---------------------------------------------------------------------------
// InitSysCtrl:
//---------------------------------------------------------------------------
// This function initializes the System Control registers to a known state.
// - Disables the watchdog
// - Set the PLLCR for proper SYSCLKOUT frequency
// - Set the pre-scaler for the high and low frequency peripheral clocks
// - Enable the clocks to the peripherals

void InitSysCtrl(void)
{
   // Disable the watchdog
   DisableDog();

   // Initialize the PLL control: PLLCR and DIVSEL
   InitPll(cpuConf.dsp28_PLLCR, cpuConf.dsp28_DIVSEL);

   // Initialize the peripheral clocks
   InitPeripheralClocks();
}

//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// InitPieCtrl: 
//---------------------------------------------------------------------------
// This function initializes the PIE control registers to a known state.
//
void InitPieCtrl(void)
{
    // Clear all interrupts and initialize PIE vector table:
    // Disable CPU interrupts
    DINT;

    // Disable the PIE
    PieCtrlRegs.PIECTRL.bit.ENPIE = 0;

    // Clear all PIEIER registers:
    PieCtrlRegs.PIEIER1.all = 0;
    PieCtrlRegs.PIEIER2.all = 0;
    PieCtrlRegs.PIEIER3.all = 0;	
    PieCtrlRegs.PIEIER4.all = 0;
    PieCtrlRegs.PIEIER5.all = 0;
    PieCtrlRegs.PIEIER6.all = 0;
    PieCtrlRegs.PIEIER7.all = 0;
    PieCtrlRegs.PIEIER8.all = 0;
    PieCtrlRegs.PIEIER9.all = 0;
    PieCtrlRegs.PIEIER10.all = 0;
    PieCtrlRegs.PIEIER11.all = 0;
    PieCtrlRegs.PIEIER12.all = 0;

    // Clear all PIEIFR registers:
    PieCtrlRegs.PIEIFR1.all = 0;
    PieCtrlRegs.PIEIFR2.all = 0;
    PieCtrlRegs.PIEIFR3.all = 0;	
    PieCtrlRegs.PIEIFR4.all = 0;
    PieCtrlRegs.PIEIFR5.all = 0;
    PieCtrlRegs.PIEIFR6.all = 0;
    PieCtrlRegs.PIEIFR7.all = 0;
    PieCtrlRegs.PIEIFR8.all = 0;
    PieCtrlRegs.PIEIFR9.all = 0;
    PieCtrlRegs.PIEIFR10.all = 0;
    PieCtrlRegs.PIEIFR11.all = 0;
    PieCtrlRegs.PIEIFR12.all = 0;
}


//---------------------------------------------------------------------------
// EnableInterrupts: 
//---------------------------------------------------------------------------
// This function enables the PIE module and CPU interrupts
//
void EnableInterrupts(void)
{
    // Enable the PIE
    PieCtrlRegs.PIECTRL.bit.ENPIE = 1;
    		
    // Enables PIE to drive a pulse into the CPU 
    PieCtrlRegs.PIEACK.all = 0xFFFF;  

    // Enable Interrupts at the CPU level 
    EINT;

    // Enable Global realtime interrupt DBGM
    ERTM;          
}

//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// ServiceDog:
//---------------------------------------------------------------------------
// This function resets the watchdog timer.
// Enable this function for using ServiceDog in the application

void ServiceDog(void)
{
    EALLOW;
    SysCtrlRegs.WDKEY = 0x0055;
    SysCtrlRegs.WDKEY = 0x00AA;
    EDIS;
}

//---------------------------------------------------------------------------
// DisableDog:
//---------------------------------------------------------------------------
// This function disables the watchdog timer.

void DisableDog(void)
{
    EALLOW;
    SysCtrlRegs.WDCR= 0x0068;
    EDIS;
}

// ----------------------------------------------------------------------------

void InitPll(Uint16 val, Uint16 divsel)
{
   volatile Uint16 iVol;

   // Make sure the PLL is not running in limp mode
   if (SysCtrlRegs.PLLSTS.bit.MCLKSTS != 0) {
      // Missing external clock has been detected
      // Replace this line with a call to an appropriate
      // SystemShutdown(); function.
      asm("        ESTOP0");
   }

   // CLKINDIV MUST be 0 before PLLCR can be changed from
   // 0x0000. It is set to 0 by an external reset XRSn
   if (SysCtrlRegs.PLLSTS.bit.DIVSEL != 0){
       EALLOW;
       SysCtrlRegs.PLLSTS.bit.DIVSEL = 0;
       EDIS;
   }

   // Change the PLLCR
   if (SysCtrlRegs.PLLCR.bit.DIV != val){
      EALLOW;
      // Before setting PLLCR turn off missing clock detect logic
      SysCtrlRegs.PLLSTS.bit.MCLKOFF = 1;
      SysCtrlRegs.PLLCR.bit.DIV = val;
      EDIS;

      // Optional: Wait for PLL to lock.
      // During this time the CPU will switch to OSCCLK/2 until
      // the PLL is stable.  Once the PLL is stable the CPU will
      // switch to the new PLL value.
      //
      // This time-to-lock is monitored by a PLL lock counter.
      //
      // Code is not required to sit and wait for the PLL to lock.
      // However, if the code does anything that is timing critical,
      // and requires the correct clock be locked, then it is best to
      // wait until this switching has completed.

      // Wait for the PLL lock bit to be set.
      // Note this bit is not available on 281x devices.  For those devices
      // use a software loop to perform the required count.

      // The watchdog should be disabled before this loop, or fed within
      // the loop via ServiceDog().

	  // Uncomment to disable the watchdog
      DisableDog();

      while(SysCtrlRegs.PLLSTS.bit.PLLLOCKS != 1){
	      // Uncomment to service the watchdog
          // ServiceDog();
      }

      EALLOW;
      SysCtrlRegs.PLLSTS.bit.MCLKOFF = 0;
      SysCtrlRegs.PLLSTS.bit.DIVSEL = divsel;
      EDIS;
    }
	/*
	// Make sure the PLL is not running in limp mode
   if (SysCtrlRegs.PLLSTS.bit.MCLKSTS != 0)
   {
      // Missing external clock has been detected
      // Replace this line with a call to an appropriate
      // SystemShutdown(); function.
      asm("        ESTOP0");
   }

   // DIVSEL MUST be 0 before PLLCR can be changed from
   // 0x0000. It is set to 0 by an external reset XRSn
   // This puts us in 1/4
   if (SysCtrlRegs.PLLSTS.bit.DIVSEL != 0)
   {
       EALLOW;
       SysCtrlRegs.PLLSTS.bit.DIVSEL = 0;
       EDIS;
   }

   // Change the PLLCR
   if (SysCtrlRegs.PLLCR.bit.DIV != val)
   {

      EALLOW;
      // Before setting PLLCR turn off missing clock detect logic
      SysCtrlRegs.PLLSTS.bit.MCLKOFF = 1;
      SysCtrlRegs.PLLCR.bit.DIV = val;
      EDIS;

      // Optional: Wait for PLL to lock.
      // During this time the CPU will switch to OSCCLK/2 until
      // the PLL is stable.  Once the PLL is stable the CPU will
      // switch to the new PLL value.
      //
      // This time-to-lock is monitored by a PLL lock counter.
      //
      // Code is not required to sit and wait for the PLL to lock.
      // However, if the code does anything that is timing critical,
      // and requires the correct clock be locked, then it is best to
      // wait until this switching has completed.

      // Wait for the PLL lock bit to be set.

      // The watchdog should be disabled before this loop, or fed within
      // the loop via ServiceDog().

	  // Uncomment to disable the watchdog
      DisableDog();

      while(SysCtrlRegs.PLLSTS.bit.PLLLOCKS != 1)
      {
	      // Uncomment to service the watchdog
          // ServiceDog();
      }

      EALLOW;
      SysCtrlRegs.PLLSTS.bit.MCLKOFF = 0;
      EDIS;
    }
        */
    // If switching to 1/2
        if((divsel == 1)||(divsel == 2)) {
                EALLOW;
            SysCtrlRegs.PLLSTS.bit.DIVSEL = divsel;
            EDIS;
	}

	// If switching to 1/1
        // * First go to 1/2 and let the power settle
        //   The time required will depend on the system, this is only an example
        // * Then switch to 1/1
        if(divsel == 3) {
                EALLOW;
            SysCtrlRegs.PLLSTS.bit.DIVSEL = 2;
            DELAY_US(50L);
            SysCtrlRegs.PLLSTS.bit.DIVSEL = 3;
            EDIS;
    }
}

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// If HISPCP(2:0) > 0, HSPCLK = SYSCLKOUT/(HISPCP X 2)
// If HISPCP = 0, HSPCLK = SYSCLKOUT
// 000 High speed clock = SYSCLKOUT/1
// 001 High speed clock = SYSCLKOUT/2 (reset default)
// 010 High speed clock = SYSCLKOUT/4
// 011 High speed clock = SYSCLKOUT/6
// 100 High speed clock = SYSCLKOUT/8
// 101 High speed clock = SYSCLKOUT/10
// 110 High speed clock = SYSCLKOUT/12
// 111 High speed clock = SYSCLKOUT/14
// If LOSPCP(2:0) > 0, LSPCLK = SYSCLKOUT/(LOSPCP X 2)
// If LOSPCP = 0, LSPCLK = SYSCLKOUT
// 000 Low speed clock = SYSCLKOUT/1
// 001 Low speed clock = SYSCLKOUT/2
// 010 Low speed clock = SYSCLKOUT/4 (reset default)
// 011 Low speed clock = SYSCLKOUT/6
// 100 Low speed clock = SYSCLKOUT/8
// 101 Low speed clock = SYSCLKOUT/10
// 110 Low speed clock = SYSCLKOUT/12
// 111 Low speed clock = SYSCLKOUT/14
// ----------------------------------------------------------------------------
void InitPeripheralClocks(void)
{
   	EALLOW;
	// PABLO, I DO NOT KNOW WHY x2 WAS USED HERE. IF WE WANT InitCPU TO RECEIVE
	// DIRECTLY THE HSPCLK and LSPCLK DIVIDERS, WE SHOULD INSTEAD DIVIDE IT BY 2
   	SysCtrlRegs.HISPCP.all = cpuConf.hspCLK > 0 ? cpuConf.hspCLK>>1 : 1;
   	SysCtrlRegs.LOSPCP.all = cpuConf.lspCLK > 0 ? cpuConf.lspCLK>>1 : 2;

	/* external interface XINTF configuration
	// XCLKOUT to SYSCLKOUT ratio.  By default XCLKOUT = 1/4 SYSCLKOUT
	// XTIMCLK = SYSCLKOUT/2
	XintfRegs.XINTCNF2.bit.XTIMCLK = 1;
	// XCLKOUT = XTIMCLK/4
	XintfRegs.XINTCNF2.bit.CLKMODE = 0;
	// Enable XCLKOUT
	XintfRegs.XINTCNF2.bit.CLKOFF = 0;
	*/

  	EDIS;
}

// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

