/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	cpu.h
 Project	:  	TIMotorLIB
 
 TI Doc:		
			
			<a href="http://www.ti.com/lit/SPRUFB0"> SPRUFB0</a>
			(TMS320x2833x, 2823x System Control and Interrupts)
			
			<a href="http://focus.ti.com/lit/ds/sprs439h/sprs439h.pdf"> SPRS439H</a> 
			(TMS320F28335, TMS320F28334, TMS320F28332
				  TMS320F28235, TMS320F28234, TMS320F28232
				  Digital Signal Controllers (DSCs)) 
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief 		CPU Clocking and Interrupts configuration macros and routines
 \date 			2007
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef CPU_H
#define CPU_H

#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)	// DSP280x Headerfile Include File
#include "IQConstants.h"				// IQMath definitions

/**-----------------------------------------------------------------------------
      Specify the PLL control register (PLLCR) and clock in divide (DIVSEL) value.

      if DIVSEL = 0,1: SYSCLKOUT = (OSCCLK * PLLCR)/4
      if DIVSEL = 2: SYSCLKOUT = (OSCCLK * PLLCR)/2
-----------------------------------------------------------------------------*/
#define OSCCLK	30000000	///< main oscillator frequency in Mhz
/*-----------------------------------------------------------------------------
      Specify the PLL control register (PLLCR) and divide select (DIVSEL) value.
-----------------------------------------------------------------------------*/
//#define DSP28_DIVSEL   0   // Enable /4 for SYSCLKOUT
//#define DSP28_DIVSEL   1 // Enable /4 for SYSCKOUT
#define DSP28_DIVSEL     2 // Enable /2 for SYSCLKOUT
//#define DSP28_DIVSEL     3 // Enable /1 for SYSCLKOUT

#define DSP28_PLLCR   10
//#define DSP28_PLLCR    9
//#define DSP28_PLLCR    8
//#define DSP28_PLLCR    7
//#define DSP28_PLLCR    6  // Uncomment for 60 MHz devices [60 MHz = (20MHz * 6)/2]
//#define DSP28_PLLCR    5
//#define DSP28_PLLCR    4
//#define DSP28_PLLCR    3
//#define DSP28_PLLCR    2
//#define DSP28_PLLCR    1
//#define DSP28_PLLCR    0  // PLL is bypassed in this mode
//----------------------------------------------------------------------------

/**-----------------------------------------------------------------------------
      Specify the clock rate of the CPU (SYSCLKOUT) in nS.

      Take into account the input clock frequency and the PLL multiplier
      selected in step 1.

      Use one of the values provided, or define your own.
      The trailing L is required tells the compiler to treat
      the number as a 64-bit value.

      Only one statement should be uncommented.

      Example:  CLKIN is a 20MHz crystal.

                In step 1 the user specified PLLCR = 0xA for a
                100Mhz CPU clock (SYSCLKOUT = 100MHz).

                In this case, the CPU_RATE will be 10.000L
                Uncomment the line:  #define CPU_RATE   10.000L

                For a 60 MHz CPU clock (SYSCLKOUT = 60MHz), user
                specified PLLCR = 0x06 in step 1.

                In this case, the CPU_RATE will be 16.667L
                Uncomment the line:  #define CPU_RATE   16.667L
-----------------------------------------------------------------------------*/
//#define CPU_RATE   6.667L    // for a 150MHz CPU clock speed (SYSCLKOUT)
//#define CPU_RATE   10.000L   // for a 100MHz CPU clock speed (SYSCLKOUT)
//#define CPU_RATE   13.330L   // for a 75MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   16.667L   // for a 60MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   20.000L   // for a 50MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   33.333L   // for a 30MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   41.667L   // for a 24MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   50.000L   // for a 20MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE   66.667L   // for a 15MHz CPU clock speed  (SYSCLKOUT)
//#define CPU_RATE  100.000L   // for a 10MHz CPU clock speed  (SYSCLKOUT)


/**-----------------------------------------------------------------------------
      Specify the SYSCLKOUT max frequency for examples - either 60 MHz
      (for 60 MHz devices only) or 100 MHz (for all other devices - this is
      the default SYSCLKOUT frequency for all examples).

      Doing so will allow the debugger to compile Example code which is dependent
      upon CPU frequency for that frequency.

      Example:  CLKIN is a 20MHz crystal.
                In step 1 the user specified PLLCR = 0xA for a 100 MHz
                CPU clock (SYSCLKOUT = 100MHz).

                Then #define CPU_FRQ_100MHZ is set as 1 (default) and
                     #define CPU_FRQ_60MHZ is set as 0 (default).

                If PLLCR = 0x06 in step 1, then:
                     #define CPU_FRQ_100MHZ is set as 0 and
                     #define CPU_FRQ_60MHZ is set as 1.
-----------------------------------------------------------------------------*/

#if DSP28_PLLCR == 10
	#define CPU_RATE   6.667L  	 // for a 150MHz CPU clock speed (SYSCLKOUT)
	//#define CPU_RATE   10.000L   // for a 100MHz CPU clock speed (SYSCLKOUT)
	#define CPU_FRQ_150MHZ   1   // 150 Mhz CPU Freq - 1 for 150 MHz devices
	#define CPU_FRQ_100MHZ   0   // 100 Mhz CPU Freq - default, 1 for 100 MHz devices
  	#define CPU_FRQ_60MHZ    0   // 60 MHz CPU Freq - 1 for 60 MHz devices
#elif DSP28_PLLCR == 6
	#define CPU_RATE   16.667L   // for a 60MHz CPU clock speed (SYSCLKOUT)
	#define CPU_FRQ_100MHZ   0   // 100 Mhz CPU Freq - default, 1 for 100 MHz devices
	#define CPU_FRQ_60MHZ    1   // 60 MHz CPU Freq - 1 for 60 MHz devices
#endif

/// \struct CPUConfiguration. 
/// \brief Store CPU high and low speed peripheral clock prescaler
struct CPUConfiguration
{
	Uint32 oscCLK;			///< Oscillator frequency
	Uint16 dsp28_DIVSEL; 	///< SYSCLKOUT divider (0,1)
	Uint16 dsp28_PLLCR;		///< PLL (0,10)
	Uint16 hspCLK;			///< High-speed peripheral clock prescaler
	Uint16 lspCLK;			///< Low-speed peripheral clock prescaler
};


extern volatile struct CPUConfiguration cpuConf;


extern void DSP28x_usDelay(Uint32 Count);

/// \todo: PABLO: All these macros MUST be replaced by static inline functions in order to get the 
/// library working with other CPU frequencies without recompiling

/// \def DELAY_US(A) DO NOT MODIFY THIS LINE.
#define DELAY_US(A)  DSP28x_usDelay(((((long double) A * 1000.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)

/*! \def US2CLOCK(A, PRRSC) us. 
 * to clock value
 */
#define US2CLOCK(A, PRRSC) ( (A / (CPU_RATE / 1000.0L) ) / PRRSC)
/*! \def CPUINMHZ() 
 * CPU frequency in Mhz.
 * \todo if DSP28_DIVSEL is set to 0 --> (/4) it does not work 
 */
#define CPUINMHZ()      (OSCCLK*DSP28_PLLCR/1000000) >> (~DSP28_DIVSEL & 0x3)
/*! \def CPUINMHZ() 
 * CPU frequency in Hz.
 * \todo if DSP28_DIVSEL is set to 0 --> (/4) it does not work
 */
#define CPUINHZ()	((Uint32)(CPUINMHZ())*(Uint32)1000000)

// same but with static inline functions

/// \todo: Documentation
static inline Uint32 Us2Clock(Uint16 us, Uint16 prrsc)
{
    return  (us / (CPU_RATE / 1000.0L) ) / prrsc;
}

//-----------------------------------------------------------------------------

/// returns cpu frequency in Mhz. It uses the information stored in cpuConf
/// \todo if dsp28_DIVSEL is set to 0 --> (/4) it does not work
static inline Uint16 CpuInMhz(void)
{
    return (cpuConf.oscCLK*cpuConf.dsp28_PLLCR/1000000) >> (~cpuConf.dsp28_DIVSEL & 0x3);
}

//-----------------------------------------------------------------------------

/// returns cpu frequency in Hz. It uses the information stored in cpuConf
/// \todo if dsp28_DIVSEL is set to 0 --> (/4) it does not work
static inline Uint32 CpuInHz(void)
{
    return ((Uint32)(CpuInMhz())*(Uint32)1000000);
}

//-----------------------------------------------------------------------------

/// returns high speed clock frequency in Mhz. It uses the information stored in cpuConf
/// \todo if dsp28_DIVSEL is set to 0 --> (/4) it does not work
static inline _iq HspCLKInMhz(void)
{
    return cpuConf.hspCLK > 0 ? _IQdiv(_IQ(CpuInMhz()), _IQ(cpuConf.hspCLK)) : _IQ(CpuInMhz());
}

//-----------------------------------------------------------------------------

/// returns low speed clock frequency in Mhz. It uses the information stored in cpuConf
/// \todo if dsp28_DIVSEL is set to 0 --> (/4) it does not work
static inline _iq LspCLKInMhz(void)
{
    return cpuConf.lspCLK > 0 ? _IQdiv(_IQ(CpuInMhz()), _IQ(cpuConf.lspCLK)) : _IQ(CpuInMhz());
}

//-----------------------------------------------------------------------------


/// \def SETIDLEMODE CPU in Idle
#define SETIDLEMODE		(SysCtrlRegs.LPMCR0.bit.LPM=0)
/// \def IDLE CPU in Idle
#define IDLE asm(" IDLE")

//-----------------------------------------------------------------------------
// 	Implemented Functions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/// Init CPU clocks (spru712d pp 47).
/// \param oscCLK Oscillator frequency in Mhz
/// \param dsp28_DIVSEL SYSCLKOUT divider
///	Values: 0, 1 (/4); 2(/2); 3(/1) 
/// \param dsp28_PLLCR Pll
/// Values: from 0 (bypass) to 10 (x10)
/// \param hspCLK High speed clock pre-scaler. This clock is the clock source for: 
///	- ADC
///	- Values: [0 (reset default=SYSCLKOUT/2), 1, 2, 4, 6, 8, 10, 12, 14]
///
///
/// \param lspCLK Low speed clock pre-scaler. This clock is the clock source for:
///	- SPI, SCI, I2C
///	- McBSP
///	- Values: [0 (reset default= SYSCLKOUT/4), 1, 2, 4, 6, 8, 10, 12, 14]
//-----------------------------------------------------------------------------
extern void InitCPU(Uint32 oscCLK, Uint16 dsp28_DIVSEL, Uint16 dsp28_PLLCR,
	Uint16 hspCLK, Uint16 lspCLK);


//---------------------------------------------------------------------------
/// This function initializes the System Control registers to a known state.
/// - Disables the watchdog
/// - Set the PLLCR for proper SYSCLKOUT frequency
/// - Set the pre-scaler for the high and low frequency peripheral clocks
/// - Enable the clocks to the peripherals
//---------------------------------------------------------------------------
extern void InitSysCtrl(void);

//---------------------------------------------------------------------------
/// This function initializes the PIE control registers to a known state.
/// All the interrupt vectors are cleared.
//---------------------------------------------------------------------------
extern void InitPieCtrl(void);

//---------------------------------------------------------------------------
/// PABLO: \todo: NOT IMPLEMENTED
//---------------------------------------------------------------------------
extern void InitPieVectTable(void);

//---------------------------------------------------------------------------
/// This function enables the PIE module and CPU interrupts
//---------------------------------------------------------------------------
extern void EnableInterrupts(void);

//---------------------------------------------------------------------------
/// This function resets the watchdog timer.
/// Enable this function for using ServiceDog in the application
//---------------------------------------------------------------------------
extern void ServiceDog(void);


//---------------------------------------------------------------------------
/// This function disables the watchdog timer.
//---------------------------------------------------------------------------
extern void DisableDog(void);

//---------------------------------------------------------------------------
/// Initializes the PLL
//---------------------------------------------------------------------------
extern void InitPll(Uint16 val, Uint16 clkindiv);

//---------------------------------------------------------------------------
/// Initializes high speed and low speed clocks
//---------------------------------------------------------------------------
extern void InitPeripheralClocks(void);

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

