/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	qep.h
 Project	:  	TIMotorLIB
 
 TI Doc:		<a href="http://www.ti.com/litv/pdf/spru790d"> SPRU790</a>
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2007
 \brief 		Encoder Routines
 \details		This unit allows to program the encoder interface and calculates
 the actual position and velocity optimized for high speed (count units each unit
 timer interrupt).
 \todo			 	- Integrate hall sensor interface
					- Incorporate low speed measurments
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef QEP_H
#define QEP_H

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)
#include "FilterAndRotations.h"
#include "Utils.h"

/// \struct QEP 
/// \brief struct used for storing QEP configuration
struct QEP {
        volatile struct EQEP_REGS* eQEPRegs;///< pointer to register (EQep1Regs/EQep2Regs)
        Uint16 lines;                                           ///< encoder lines
        Uint16 lastPos;                                         ///< last Position (lines)
        Uint16 polePairs;                                       ///< pole pairs
        _iq theta_mech;                                         ///< mech. position (rad)
        _iq theta_elec;                                         ///< electrical position (rad)
	_iq w_mech;						///< mech. velocity (rad/s)
	_iq w_elec;						///< elec. velocity (rad/s)
	//_iq w_elec_lpf;					//! elec. velocity low pass filtered (rad/s)
	_iq lines2Hz;						///< convert lines to Hz
};

extern volatile struct QEP qep1;
extern volatile struct QEP qep2;

/// select position counter reset mode
typedef enum {
	PCRM_IE,		///< position counter reset on index event
	PCRM_MAX_POS,	///< position counter reset on the maximum position
	PCRM_FIE,		///< position counter reset on the first index event
	PCRM_UTE		///< position counter reset on a unit time event
	} PCRM;


//-----------------------------------------------------------------------------
// Implemented Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Initialize encoder reading (QEP1)
///	\param interruptFCN function to be called each qep_us
///  	\param qep_us unit timer
///  	\param pcrm position counter reset mode (PCRM_IE/PCRM_MAX_POS/PCRM_FIE/PCRM_UTE)
///  	\param lines encoder lines
///  	\param polePairs machine pole pairs
//-----------------------------------------------------------------------------
extern void ConfigureQEP1(void(*interruptFCN)(void), Uint16 qep_us, 
	PCRM pcrm, Uint16 lines, Uint16 polePairs);
	
//-----------------------------------------------------------------------------
/// Initialize encoder reading (QEP2)
///	\param interruptFCN function to be called each qep_us
///  	\param qep_us unit timer
///  	\param pcrm position counter reset mode
///  	\param lines encoder lines
///  	\param polePairs machine pole pairs
//-----------------------------------------------------------------------------
extern void ConfigureQEP2(void(*interruptFCN)(void), Uint16 qep_us, 
	PCRM pcrm, Uint16 lines, Uint16 polePairs);
	

//-----------------------------------------------------------------------------
/// Init GPIO ports as needed for encoder connection QEP1 (GPIO20, GPIO21, GPIO23)
//-----------------------------------------------------------------------------
extern void InitEQep1Gpio(void);


//-----------------------------------------------------------------------------
/// Init GPIO ports as needed for encoder connection QEP2 (GPIO24, GPIO25, GPIO26)
//-----------------------------------------------------------------------------
extern void InitEQep2Gpio(void);

//-----------------------------------------------------------------------------
/// Swaps direction of positive counting
///     \param pointer to qep unit (qep1/qep2)
//-----------------------------------------------------------------------------
extern void SwapDirection(volatile struct QEP *qep);

//-----------------------------------------------------------------------------
/// Calculate position and velocity from encoder readings QEP1:
///     - Mechanical position is stored in QEPx.theta_mech.
///     - Electrical position is stored in QEPx.theta_elec.
///	- Mechanical velocity is stored in QEPx.w_mech.
///	- Electrical velocity  is stored in QEPx.w_elec.
/// \warning: If ISR for encoder is used, this function MUST never be called
///	outside the ISR. This is because the function does not assume an ISR is
/// used and it also clears the unit timer flag (EQep1Regs.QCLR.bit.UTO=1;)
//-----------------------------------------------------------------------------
static inline void QEP1VelocityAndPosition(void);

//-----------------------------------------------------------------------------
/// Calculate position and velocity from encoder readings QEP2:
///	- Mechanical position is stored in QEPx.theta_mech.
///	- Electrical position is stored in QEPx.theta_elec.
///	- Mechanical velocity is stored in QEPx.w_mech.
///	- Electrical velocity  is stored in QEPx.w_elec.
/// \warning: If ISR for encoder is used, this function MUST never be called
///	outside the ISR. This is because the function does not assume an ISR is
/// used and it also clears the unit timer flag (EQep1Regs.QCLR.bit.UTO=1;)
//-----------------------------------------------------------------------------
static inline void QEP2VelocityAndPosition(void);

//-----------------------------------------------------------------------------
/// Internal function for calculation of velocity and position. It is called
/// in QEPxVelocityAndPosition.
///	\param newPos position in current sample time from QPOSLAT
/// 	\param oldPos position stored in lastPos: \todo: This parameter should be gone
/// 	\param direction direction of counting (0:down, 1:up)
/// 	\param qep pointer to configuration struct for encoder device
//-----------------------------------------------------------------------------
static inline void QEPVelocityAndPosition(Uint16 newPos, Uint16 oldPos, 
	Uint16 direction, volatile struct QEP* qep);

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Enable QEP interface
/// \param qep pointer to QEP struct which interface wants to be enabled
//-----------------------------------------------------------------------------
static inline void EnableQEP(volatile struct QEP* qep);

//-----------------------------------------------------------------------------
/// Enable QEP1 interface
//-----------------------------------------------------------------------------
static inline void EnableQEP1(void);

//-----------------------------------------------------------------------------
/// Enable QEP2 interface
//-----------------------------------------------------------------------------
static inline void EnableQEP2(void);

//-----------------------------------------------------------------------------
/// ACK interrupts from Unit timer associated to eQEPRegs register
/// \param eQEPRegs register register which interrupt is ACK
//-----------------------------------------------------------------------------
static inline void EqepPIEAck(volatile struct EQEP_REGS* eQEPRegs);

//-----------------------------------------------------------------------------
///	ACK unit timer interrupt for EQEP1. This function MUST be called if more interrupts
/// from PIEACK_GROUP5 are wanted
//-----------------------------------------------------------------------------
static inline void Eqep1PIEAck(void);

//-----------------------------------------------------------------------------
///	ACK unit timer interrupt for EQEP2. This function MUST be called if more interrupts
/// from PIEACK_GROUP5 are wanted
//-----------------------------------------------------------------------------
static inline void Eqep2PIEAck(void);


//-----------------------------------------------------------------------------
// Inline function implementation
//-----------------------------------------------------------------------------

/// \todo Same function can be used for both QEP1 and QEP2. Code for function
///	QEPVelocityAndPosition just merged in the new common one.
static inline void QEP1VelocityAndPosition(void)
{
	// position loop
	Uint16 direction = EQep1Regs.QEPSTS.bit.QDF;	// Motor direction: 0=CCW/reverse, 1=CW/forward
	Uint16 newPos = (Uint16)(EQep1Regs.QPOSCNT);	// position counter
	Uint16 oldPos;
	//**** High Speed Calculation using QEP Position counter ****//
	// Check unit Time out-event for speed calculation: (velocity loop under sampling)
	if(EQep1Regs.QFLG.bit.UTO==1){		// If unit timeout
		newPos=(Uint16)EQep1Regs.QPOSLAT;	      // Latched POSCNT value
		oldPos=qep1.lastPos;						// Latched POSCNT value
		//oldPos=(Uint16)EQep1Regs.QPOSLAT;	      // Latched POSCNT value
		QEPVelocityAndPosition(newPos, oldPos, direction, &qep1);		
		EQep1Regs.QCLR.bit.UTO=1;		 	// Clear interrupt flag
	}	
	// PABLO: This should be ONLY calculated within the bandwidth of the UTO
	/*qep1.theta_mech = _IQmpy(TWOPI,_IQ18toIQ(_IQ18(newPos)/(qep1.lines << 2)));
	WRAP2PI(qep1.theta_mech);
	qep1.theta_elec = qep1.theta_mech*qep1.polePairs;
	while (qep1.theta_elec>PI){
			qep1.theta_elec=qep1.theta_elec-TWOPI;
	}
	while (qep1.theta_elec<-PI){
			qep1.theta_elec=qep1.theta_elec+TWOPI;
	}*/
}

// ----------------------------------------------------------------------------

static inline void QEP2VelocityAndPosition(void)
{
	// position loop
	Uint16 direction = EQep2Regs.QEPSTS.bit.QDF;// Motor direction: 0=CCW/reverse, 1=CW/forward
	Uint16 newPos = (Uint16)(EQep2Regs.QPOSCNT);	// position counter
	Uint16 oldPos;
	// velocity loop under sampling	
	if(EQep2Regs.QFLG.bit.UTO==1){          // If unit timeout
		newPos=(Uint16)EQep2Regs.QPOSLAT;	      // Latched POSCNT value
		oldPos=qep2.lastPos;						// Latched POSCNT value
		//oldPos=(Uint16)EQep2Regs.QPOSLAT;	      // Latched POSCNT value
		QEPVelocityAndPosition(newPos, oldPos, direction, &qep2);		
		EQep2Regs.QCLR.bit.UTO=1;		 	// Clear interrupt flag
	}
	// PABLO: This should be ONLY calculated within the bandwidth of the UTO
	/*qep2.theta_mech = _IQmpy(TWOPI,_IQ18toIQ(_IQ18(newPos)/(qep2.lines << 2)));
	WRAP2PI(qep2.theta_mech);
	qep2.theta_elec = qep2.theta_mech*qep2.polePairs;
	while (qep2.theta_elec>PI){
			qep2.theta_elec=qep2.theta_elec-TWOPI;
	}
	while (qep2.theta_elec<-PI){
			qep2.theta_elec=qep2.theta_elec+TWOPI;
	}*/
}

// ----------------------------------------------------------------------------

static inline void QEPVelocityAndPosition(Uint16 newPos, Uint16 oldPos, 
        Uint16 direction, volatile struct QEP* qep)
{
        int16 diff_pulses;
        //GpioDataRegs.GPATOGGLE.bit.GPIO18 = 1;
        if(direction==0){               // counting down
                if(newPos>oldPos)       // counter reset
			diff_pulses = newPos-oldPos-(qep->lines<<2);
		else
			diff_pulses = newPos-oldPos;
	}
	else if(direction==1){	// counting up
		if(newPos<oldPos)	// counter reset
			diff_pulses = newPos-oldPos+(qep->lines<<2);
                else
                        diff_pulses = newPos-oldPos;                    
        }
        qep->w_mech = _IQmpy(TWOPI,_IQmpy(_IQ(diff_pulses),qep->lines2Hz));
        qep->w_elec = qep->w_mech*qep->polePairs;
        // low-pass filter velocity estimation
	// PABLO: This filter need to be tested and make it configurable. 
	//	Also, variables qepConf->w_elec and qepConf->w_elec_lpf MUST have memory
	//FILTER_LPF(qep->w_elec,qep->w_elec_lpf,_IQ(0.03046),_IQ(0.93908));
	qep->theta_mech = _IQmpy(TWOPI,_IQ18toIQ(_IQ18(newPos)/(qep->lines << 2)));
	WRAP2PI(qep->theta_mech);
	qep->theta_elec = qep->theta_mech*qep->polePairs;
	while (qep->theta_elec>PI){
			qep->theta_elec=qep->theta_elec-TWOPI;
	}
	while (qep->theta_elec<-PI){
			qep->theta_elec=qep->theta_elec+TWOPI;
	}
	// update values
	qep->lastPos = newPos;
}

// ----------------------------------------------------------------------------

static inline void EnableQEP(volatile struct QEP* qep)
{
	qep->eQEPRegs->QEPCTL.bit.QPEN=1; 			// QEP enable
}

//-----------------------------------------------------------------------------

static inline void EnableQEP1(void)
{
	EnableQEP(&qep1); 			// QEP enable
}

//-----------------------------------------------------------------------------

static inline void EnableQEP2(void)
{
	EnableQEP(&qep2); 			// QEP enable
}

//-----------------------------------------------------------------------------

static inline void EqepPIEAck(volatile struct EQEP_REGS* eQEPRegs)
{
  	// Acknowledge this interrupt to receive more interrupts from group 5
   	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP5;
	// clear all interrupt flags
	eQEPRegs->QCLR.bit.INT=1;			
	// clear interrupt flag	
	eQEPRegs->QCLR.bit.UTO=1;
}

//-----------------------------------------------------------------------------

static inline void Eqep1PIEAck(void)
{
	EqepPIEAck(qep1.eQEPRegs);
}

//-----------------------------------------------------------------------------

static inline void Eqep2PIEAck(void)
{
	EqepPIEAck(qep2.eQEPRegs);
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

